# This file was auto generated by Styx.
# Do not edit this file directly.

import typing
import pathlib
from styxdefs import *

RESPONSEMEAN_METADATA = Metadata(
    id="580e677d66de76d497b1d37c93d76dd82a169d45.boutiques",
    name="responsemean",
    package="mrtrix",
    container_image_tag="mrtrix3/mrtrix3:3.0.4",
)


_ResponsemeanParamsDictNoTag = typing.TypedDict('_ResponsemeanParamsDictNoTag', {
    "input_response": list[InputPathType],
    "output_response": str,
    "legacy": bool,
    "nocleanup": bool,
    "scratch_dir": typing.NotRequired[InputPathType | None],
    "continue_scratch_dir": typing.NotRequired[list[InputPathType] | None],
    "info": bool,
    "quiet": bool,
    "debug": bool,
    "force": bool,
    "nthreads": typing.NotRequired[float | None],
    "config": typing.NotRequired[list[str] | None],
    "help": bool,
    "version": bool,
})
ResponsemeanParamsDictTagged = typing.TypedDict('ResponsemeanParamsDictTagged', {
    "@type": typing.Literal["mrtrix/responsemean"],
    "input_response": list[InputPathType],
    "output_response": str,
    "legacy": bool,
    "nocleanup": bool,
    "scratch_dir": typing.NotRequired[InputPathType | None],
    "continue_scratch_dir": typing.NotRequired[list[InputPathType] | None],
    "info": bool,
    "quiet": bool,
    "debug": bool,
    "force": bool,
    "nthreads": typing.NotRequired[float | None],
    "config": typing.NotRequired[list[str] | None],
    "help": bool,
    "version": bool,
})
ResponsemeanParamsDict = _ResponsemeanParamsDictNoTag | ResponsemeanParamsDictTagged


class ResponsemeanOutputs(typing.NamedTuple):
    """
    Output object returned when calling `ResponsemeanParamsDict(...)`.
    """
    root: OutputPathType
    """Output root folder. This is the root folder for all outputs."""
    output_response_file: OutputPathType
    """File containing the averaged tissue response function"""


def responsemean_params(
    input_response: list[InputPathType],
    output_response: str,
    legacy: bool = False,
    nocleanup: bool = False,
    scratch_dir: InputPathType | None = None,
    continue_scratch_dir: list[InputPathType] | None = None,
    info: bool = False,
    quiet: bool = False,
    debug: bool = False,
    force: bool = False,
    nthreads: float | None = None,
    config: list[str] | None = None,
    help_: bool = False,
    version: bool = False,
) -> ResponsemeanParamsDictTagged:
    """
    Build parameters.
    
    Args:
        input_response: Input response functions.
        output_response: Output mean response function.
        legacy: Calculate the mean response function from a set of text files.
        nocleanup: Do not delete intermediate files during script execution,\
            and do not delete scratch directory at script completion.
        scratch_dir: Manually specify the path in which to generate the scratch\
            directory.
        continue_scratch_dir: Continue the script from a previous execution;\
            must provide the scratch directory path.
        info: Display information messages.
        quiet: Do not display information messages or progress status.
        debug: Display debugging messages.
        force: Force overwrite of output files.
        nthreads: Use this number of threads in multi-threaded applications\
            (set to 0 to disable multi-threading).
        config: Temporarily set the value of an MRtrix config file entry.
        help_: Display help information and exit.
        version: Display version information and exit.
    Returns:
        Parameter dictionary
    """
    params = {
        "@type": "mrtrix/responsemean",
        "input_response": input_response,
        "output_response": output_response,
        "legacy": legacy,
        "nocleanup": nocleanup,
        "info": info,
        "quiet": quiet,
        "debug": debug,
        "force": force,
        "help": help_,
        "version": version,
    }
    if scratch_dir is not None:
        params["scratch_dir"] = scratch_dir
    if continue_scratch_dir is not None:
        params["continue_scratch_dir"] = continue_scratch_dir
    if nthreads is not None:
        params["nthreads"] = nthreads
    if config is not None:
        params["config"] = config
    return params


def responsemean_validate(
    params: typing.Any,
) -> None:
    """
    Validate parameters. Throws an error if `params` is not a valid
    `ResponsemeanParamsDict` object.
    
    Args:
        params: The parameters object to validate.
    """
    if params is None or not isinstance(params, dict):
        raise StyxValidationError(f'Params object has the wrong type \'{type(params)}\'')
    if params.get("input_response", None) is None:
        raise StyxValidationError("`input_response` must not be None")
    if not isinstance(params["input_response"], list):
        raise StyxValidationError(f'`input_response` has the wrong type: Received `{type(params.get("input_response", None))}` expected `list[InputPathType]`')
    for e in params["input_response"]:
        if not isinstance(e, (pathlib.Path, str)):
            raise StyxValidationError(f'`input_response` has the wrong type: Received `{type(params.get("input_response", None))}` expected `list[InputPathType]`')
    if params.get("output_response", None) is None:
        raise StyxValidationError("`output_response` must not be None")
    if not isinstance(params["output_response"], str):
        raise StyxValidationError(f'`output_response` has the wrong type: Received `{type(params.get("output_response", None))}` expected `str`')
    if params.get("legacy", False) is None:
        raise StyxValidationError("`legacy` must not be None")
    if not isinstance(params["legacy"], bool):
        raise StyxValidationError(f'`legacy` has the wrong type: Received `{type(params.get("legacy", False))}` expected `bool`')
    if params.get("nocleanup", False) is None:
        raise StyxValidationError("`nocleanup` must not be None")
    if not isinstance(params["nocleanup"], bool):
        raise StyxValidationError(f'`nocleanup` has the wrong type: Received `{type(params.get("nocleanup", False))}` expected `bool`')
    if params.get("scratch_dir", None) is not None:
        if not isinstance(params["scratch_dir"], (pathlib.Path, str)):
            raise StyxValidationError(f'`scratch_dir` has the wrong type: Received `{type(params.get("scratch_dir", None))}` expected `InputPathType | None`')
    if params.get("continue_scratch_dir", None) is not None:
        if not isinstance(params["continue_scratch_dir"], list):
            raise StyxValidationError(f'`continue_scratch_dir` has the wrong type: Received `{type(params.get("continue_scratch_dir", None))}` expected `list[InputPathType] | None`')
        if len(params["continue_scratch_dir"]) != 2:
            raise StyxValidationError("Parameter `continue_scratch_dir` must contain exactly 2 elements")
        for e in params["continue_scratch_dir"]:
            if not isinstance(e, (pathlib.Path, str)):
                raise StyxValidationError(f'`continue_scratch_dir` has the wrong type: Received `{type(params.get("continue_scratch_dir", None))}` expected `list[InputPathType] | None`')
    if params.get("info", False) is None:
        raise StyxValidationError("`info` must not be None")
    if not isinstance(params["info"], bool):
        raise StyxValidationError(f'`info` has the wrong type: Received `{type(params.get("info", False))}` expected `bool`')
    if params.get("quiet", False) is None:
        raise StyxValidationError("`quiet` must not be None")
    if not isinstance(params["quiet"], bool):
        raise StyxValidationError(f'`quiet` has the wrong type: Received `{type(params.get("quiet", False))}` expected `bool`')
    if params.get("debug", False) is None:
        raise StyxValidationError("`debug` must not be None")
    if not isinstance(params["debug"], bool):
        raise StyxValidationError(f'`debug` has the wrong type: Received `{type(params.get("debug", False))}` expected `bool`')
    if params.get("force", False) is None:
        raise StyxValidationError("`force` must not be None")
    if not isinstance(params["force"], bool):
        raise StyxValidationError(f'`force` has the wrong type: Received `{type(params.get("force", False))}` expected `bool`')
    if params.get("nthreads", None) is not None:
        if not isinstance(params["nthreads"], (float, int)):
            raise StyxValidationError(f'`nthreads` has the wrong type: Received `{type(params.get("nthreads", None))}` expected `float | None`')
    if params.get("config", None) is not None:
        if not isinstance(params["config"], list):
            raise StyxValidationError(f'`config` has the wrong type: Received `{type(params.get("config", None))}` expected `list[str] | None`')
        for e in params["config"]:
            if not isinstance(e, str):
                raise StyxValidationError(f'`config` has the wrong type: Received `{type(params.get("config", None))}` expected `list[str] | None`')
    if params.get("help", False) is None:
        raise StyxValidationError("`help` must not be None")
    if not isinstance(params["help"], bool):
        raise StyxValidationError(f'`help` has the wrong type: Received `{type(params.get("help", False))}` expected `bool`')
    if params.get("version", False) is None:
        raise StyxValidationError("`version` must not be None")
    if not isinstance(params["version"], bool):
        raise StyxValidationError(f'`version` has the wrong type: Received `{type(params.get("version", False))}` expected `bool`')


def responsemean_cargs(
    params: ResponsemeanParamsDict,
    execution: Execution,
) -> list[str]:
    """
    Build command-line arguments from parameters.
    
    Args:
        params: The parameters.
        execution: The execution object for resolving input paths.
    Returns:
        Command-line arguments.
    """
    cargs = []
    cargs.append("responsemean")
    cargs.extend([execution.input_file(f) for f in params.get("input_response", None)])
    cargs.append(params.get("output_response", None))
    if params.get("legacy", False):
        cargs.append("-legacy")
    if params.get("nocleanup", False):
        cargs.append("-nocleanup")
    if params.get("scratch_dir", None) is not None:
        cargs.extend([
            "-scratch",
            execution.input_file(params.get("scratch_dir", None))
        ])
    if params.get("continue_scratch_dir", None) is not None:
        cargs.extend([
            "-continue",
            *[execution.input_file(f) for f in params.get("continue_scratch_dir", None)]
        ])
    if params.get("info", False):
        cargs.append("-info")
    if params.get("quiet", False):
        cargs.append("-quiet")
    if params.get("debug", False):
        cargs.append("-debug")
    if params.get("force", False):
        cargs.append("-force")
    if params.get("nthreads", None) is not None:
        cargs.extend([
            "-nthreads",
            str(params.get("nthreads", None))
        ])
    if params.get("config", None) is not None:
        cargs.extend([
            "-config",
            *params.get("config", None)
        ])
    if params.get("help", False):
        cargs.append("-help")
    if params.get("version", False):
        cargs.append("-version")
    return cargs


def responsemean_outputs(
    params: ResponsemeanParamsDict,
    execution: Execution,
) -> ResponsemeanOutputs:
    """
    Build outputs object containing output file paths and possibly stdout/stderr.
    
    Args:
        params: The parameters.
        execution: The execution object for resolving input paths.
    Returns:
        Outputs object.
    """
    ret = ResponsemeanOutputs(
        root=execution.output_file("."),
        output_response_file=execution.output_file(params.get("output_response", None)),
    )
    return ret


def responsemean_execute(
    params: ResponsemeanParamsDict,
    runner: Runner | None = None,
) -> ResponsemeanOutputs:
    """
    responsemean
    
    Calculate the mean response function from a set of text files.
    
    Author: MRTrix3 Developers
    
    URL: https://www.mrtrix.org/
    
    Args:
        params: The parameters.
        runner: Command runner.
    Returns:
        NamedTuple of outputs (described in `ResponsemeanOutputs`).
    """
    responsemean_validate(params)
    runner = runner or get_global_runner()
    execution = runner.start_execution(RESPONSEMEAN_METADATA)
    params = execution.params(params)
    cargs = responsemean_cargs(params, execution)
    ret = responsemean_outputs(params, execution)
    execution.run(cargs)
    return ret


def responsemean(
    input_response: list[InputPathType],
    output_response: str,
    legacy: bool = False,
    nocleanup: bool = False,
    scratch_dir: InputPathType | None = None,
    continue_scratch_dir: list[InputPathType] | None = None,
    info: bool = False,
    quiet: bool = False,
    debug: bool = False,
    force: bool = False,
    nthreads: float | None = None,
    config: list[str] | None = None,
    help_: bool = False,
    version: bool = False,
    runner: Runner | None = None,
) -> ResponsemeanOutputs:
    """
    responsemean
    
    Calculate the mean response function from a set of text files.
    
    Author: MRTrix3 Developers
    
    URL: https://www.mrtrix.org/
    
    Args:
        input_response: Input response functions.
        output_response: Output mean response function.
        legacy: Calculate the mean response function from a set of text files.
        nocleanup: Do not delete intermediate files during script execution,\
            and do not delete scratch directory at script completion.
        scratch_dir: Manually specify the path in which to generate the scratch\
            directory.
        continue_scratch_dir: Continue the script from a previous execution;\
            must provide the scratch directory path.
        info: Display information messages.
        quiet: Do not display information messages or progress status.
        debug: Display debugging messages.
        force: Force overwrite of output files.
        nthreads: Use this number of threads in multi-threaded applications\
            (set to 0 to disable multi-threading).
        config: Temporarily set the value of an MRtrix config file entry.
        help_: Display help information and exit.
        version: Display version information and exit.
        runner: Command runner.
    Returns:
        NamedTuple of outputs (described in `ResponsemeanOutputs`).
    """
    params = responsemean_params(
        input_response=input_response,
        output_response=output_response,
        legacy=legacy,
        nocleanup=nocleanup,
        scratch_dir=scratch_dir,
        continue_scratch_dir=continue_scratch_dir,
        info=info,
        quiet=quiet,
        debug=debug,
        force=force,
        nthreads=nthreads,
        config=config,
        help_=help_,
        version=version,
    )
    return responsemean_execute(params, runner)


__all__ = [
    "RESPONSEMEAN_METADATA",
    "ResponsemeanOutputs",
    "ResponsemeanParamsDict",
    "ResponsemeanParamsDictTagged",
    "responsemean",
    "responsemean_execute",
    "responsemean_params",
]



# This file was auto generated by Styx.
# Do not edit this file directly.

import typing
import pathlib
from styxdefs import *

TCKSIFT_METADATA = Metadata(
    id="7ab67267156023e9057e22433c919ff074901975.boutiques",
    name="tcksift",
    package="mrtrix",
    container_image_tag="mrtrix3/mrtrix3:3.0.4",
)


_TcksiftConfigParamsDictNoTag = typing.TypedDict('_TcksiftConfigParamsDictNoTag', {
    "key": str,
    "value": str,
})
TcksiftConfigParamsDictTagged = typing.TypedDict('TcksiftConfigParamsDictTagged', {
    "@type": typing.Literal["config"],
    "key": str,
    "value": str,
})
TcksiftConfigParamsDict = _TcksiftConfigParamsDictNoTag | TcksiftConfigParamsDictTagged


_TcksiftParamsDictNoTag = typing.TypedDict('_TcksiftParamsDictNoTag', {
    "nofilter": bool,
    "output_at_counts": typing.NotRequired[list[int] | None],
    "proc_mask": typing.NotRequired[InputPathType | None],
    "act": typing.NotRequired[InputPathType | None],
    "fd_scale_gm": bool,
    "no_dilate_lut": bool,
    "make_null_lobes": bool,
    "remove_untracked": bool,
    "fd_thresh": typing.NotRequired[float | None],
    "csv": typing.NotRequired[str | None],
    "out_mu": typing.NotRequired[str | None],
    "output_debug": bool,
    "out_selection": typing.NotRequired[str | None],
    "term_number": typing.NotRequired[int | None],
    "term_ratio": typing.NotRequired[float | None],
    "term_mu": typing.NotRequired[float | None],
    "info": bool,
    "quiet": bool,
    "debug": bool,
    "force": bool,
    "nthreads": typing.NotRequired[int | None],
    "config": typing.NotRequired[list[TcksiftConfigParamsDict] | None],
    "help": bool,
    "version": bool,
    "in_tracks": InputPathType,
    "in_fod": InputPathType,
    "out_tracks": str,
})
TcksiftParamsDictTagged = typing.TypedDict('TcksiftParamsDictTagged', {
    "@type": typing.Literal["mrtrix/tcksift"],
    "nofilter": bool,
    "output_at_counts": typing.NotRequired[list[int] | None],
    "proc_mask": typing.NotRequired[InputPathType | None],
    "act": typing.NotRequired[InputPathType | None],
    "fd_scale_gm": bool,
    "no_dilate_lut": bool,
    "make_null_lobes": bool,
    "remove_untracked": bool,
    "fd_thresh": typing.NotRequired[float | None],
    "csv": typing.NotRequired[str | None],
    "out_mu": typing.NotRequired[str | None],
    "output_debug": bool,
    "out_selection": typing.NotRequired[str | None],
    "term_number": typing.NotRequired[int | None],
    "term_ratio": typing.NotRequired[float | None],
    "term_mu": typing.NotRequired[float | None],
    "info": bool,
    "quiet": bool,
    "debug": bool,
    "force": bool,
    "nthreads": typing.NotRequired[int | None],
    "config": typing.NotRequired[list[TcksiftConfigParamsDict] | None],
    "help": bool,
    "version": bool,
    "in_tracks": InputPathType,
    "in_fod": InputPathType,
    "out_tracks": str,
})
TcksiftParamsDict = _TcksiftParamsDictNoTag | TcksiftParamsDictTagged


def tcksift_config(
    key: str,
    value: str,
) -> TcksiftConfigParamsDictTagged:
    """
    Build parameters.
    
    Args:
        key: temporarily set the value of an MRtrix config file entry.
        value: temporarily set the value of an MRtrix config file entry.
    Returns:
        Parameter dictionary
    """
    params = {
        "@type": "config",
        "key": key,
        "value": value,
    }
    return params


def tcksift_config_validate(
    params: typing.Any,
) -> None:
    """
    Validate parameters. Throws an error if `params` is not a valid
    `TcksiftConfigParamsDict` object.
    
    Args:
        params: The parameters object to validate.
    """
    if params is None or not isinstance(params, dict):
        raise StyxValidationError(f'Params object has the wrong type \'{type(params)}\'')
    if params.get("key", None) is None:
        raise StyxValidationError("`key` must not be None")
    if not isinstance(params["key"], str):
        raise StyxValidationError(f'`key` has the wrong type: Received `{type(params.get("key", None))}` expected `str`')
    if params.get("value", None) is None:
        raise StyxValidationError("`value` must not be None")
    if not isinstance(params["value"], str):
        raise StyxValidationError(f'`value` has the wrong type: Received `{type(params.get("value", None))}` expected `str`')


def tcksift_config_cargs(
    params: TcksiftConfigParamsDict,
    execution: Execution,
) -> list[str]:
    """
    Build command-line arguments from parameters.
    
    Args:
        params: The parameters.
        execution: The execution object for resolving input paths.
    Returns:
        Command-line arguments.
    """
    cargs = []
    cargs.append("-config")
    cargs.append(params.get("key", None))
    cargs.append(params.get("value", None))
    return cargs


class TcksiftOutputs(typing.NamedTuple):
    """
    Output object returned when calling `TcksiftParamsDict(...)`.
    """
    root: OutputPathType
    """Output root folder. This is the root folder for all outputs."""
    out_tracks: OutputPathType
    """the output filtered tracks file"""
    csv_: OutputPathType | None
    """output statistics of execution per iteration to a .csv file """
    out_mu: OutputPathType | None
    """output the final value of SIFT proportionality coefficient mu to a text
    file """
    out_selection: OutputPathType | None
    """output a text file containing the binary selection of streamlines """


def tcksift_params(
    in_tracks: InputPathType,
    in_fod: InputPathType,
    out_tracks: str,
    nofilter: bool = False,
    output_at_counts: list[int] | None = None,
    proc_mask: InputPathType | None = None,
    act: InputPathType | None = None,
    fd_scale_gm: bool = False,
    no_dilate_lut: bool = False,
    make_null_lobes: bool = False,
    remove_untracked: bool = False,
    fd_thresh: float | None = None,
    csv_: str | None = None,
    out_mu: str | None = None,
    output_debug: bool = False,
    out_selection: str | None = None,
    term_number: int | None = None,
    term_ratio: float | None = None,
    term_mu: float | None = None,
    info: bool = False,
    quiet: bool = False,
    debug: bool = False,
    force: bool = False,
    nthreads: int | None = None,
    config: list[TcksiftConfigParamsDict] | None = None,
    help_: bool = False,
    version: bool = False,
) -> TcksiftParamsDictTagged:
    """
    Build parameters.
    
    Args:
        in_tracks: the input track file.
        in_fod: input image containing the spherical harmonics of the fibre\
            orientation distributions.
        out_tracks: the output filtered tracks file.
        nofilter: do NOT perform track filtering - just construct the model in\
            order to provide output debugging images.
        output_at_counts: output filtered track files (and optionally debugging\
            images if -output_debug is specified) at specific numbers of remaining\
            streamlines; provide as comma-separated list of integers.
        proc_mask: provide an image containing the processing mask weights for\
            the model; image spatial dimensions must match the fixel image.
        act: use an ACT five-tissue-type segmented anatomical image to derive\
            the processing mask.
        fd_scale_gm: provide this option (in conjunction with -act) to\
            heuristically downsize the fibre density estimates based on the\
            presence of GM in the voxel. This can assist in reducing tissue\
            interface effects when using a single-tissue deconvolution algorithm.
        no_dilate_lut: do NOT dilate FOD lobe lookup tables; only map\
            streamlines to FOD lobes if the precise tangent lies within the angular\
            spread of that lobe.
        make_null_lobes: add an additional FOD lobe to each voxel, with zero\
            integral, that covers all directions with zero / negative FOD\
            amplitudes.
        remove_untracked: remove FOD lobes that do not have any streamline\
            density attributed to them; this improves filtering slightly, at the\
            expense of longer computation time (and you can no longer do\
            quantitative comparisons between reconstructions if this is enabled).
        fd_thresh: fibre density threshold; exclude an FOD lobe from filtering\
            processing if its integral is less than this amount (streamlines will\
            still be mapped to it, but it will not contribute to the cost function\
            or the filtering).
        csv_: output statistics of execution per iteration to a .csv file.
        out_mu: output the final value of SIFT proportionality coefficient mu\
            to a text file.
        output_debug: provide various output images for assessing & debugging\
            performance etc.
        out_selection: output a text file containing the binary selection of\
            streamlines.
        term_number: number of streamlines - continue filtering until this\
            number of streamlines remain.
        term_ratio: termination ratio - defined as the ratio between reduction\
            in cost function, and reduction in density of streamlines.\
            Smaller values result in more streamlines being filtered out.
        term_mu: terminate filtering once the SIFT proportionality coefficient\
            reaches a given value.
        info: display information messages.
        quiet: do not display information messages or progress status;\
            alternatively, this can be achieved by setting the MRTRIX_QUIET\
            environment variable to a non-empty string.
        debug: display debugging messages.
        force: force overwrite of output files (caution: using the same file as\
            input and output might cause unexpected behaviour).
        nthreads: use this number of threads in multi-threaded applications\
            (set to 0 to disable multi-threading).
        config: temporarily set the value of an MRtrix config file entry.
        help_: display this information page and exit.
        version: display version information and exit.
    Returns:
        Parameter dictionary
    """
    params = {
        "@type": "mrtrix/tcksift",
        "nofilter": nofilter,
        "fd_scale_gm": fd_scale_gm,
        "no_dilate_lut": no_dilate_lut,
        "make_null_lobes": make_null_lobes,
        "remove_untracked": remove_untracked,
        "output_debug": output_debug,
        "info": info,
        "quiet": quiet,
        "debug": debug,
        "force": force,
        "help": help_,
        "version": version,
        "in_tracks": in_tracks,
        "in_fod": in_fod,
        "out_tracks": out_tracks,
    }
    if output_at_counts is not None:
        params["output_at_counts"] = output_at_counts
    if proc_mask is not None:
        params["proc_mask"] = proc_mask
    if act is not None:
        params["act"] = act
    if fd_thresh is not None:
        params["fd_thresh"] = fd_thresh
    if csv_ is not None:
        params["csv"] = csv_
    if out_mu is not None:
        params["out_mu"] = out_mu
    if out_selection is not None:
        params["out_selection"] = out_selection
    if term_number is not None:
        params["term_number"] = term_number
    if term_ratio is not None:
        params["term_ratio"] = term_ratio
    if term_mu is not None:
        params["term_mu"] = term_mu
    if nthreads is not None:
        params["nthreads"] = nthreads
    if config is not None:
        params["config"] = config
    return params


def tcksift_validate(
    params: typing.Any,
) -> None:
    """
    Validate parameters. Throws an error if `params` is not a valid
    `TcksiftParamsDict` object.
    
    Args:
        params: The parameters object to validate.
    """
    if params is None or not isinstance(params, dict):
        raise StyxValidationError(f'Params object has the wrong type \'{type(params)}\'')
    if params.get("nofilter", False) is None:
        raise StyxValidationError("`nofilter` must not be None")
    if not isinstance(params["nofilter"], bool):
        raise StyxValidationError(f'`nofilter` has the wrong type: Received `{type(params.get("nofilter", False))}` expected `bool`')
    if params.get("output_at_counts", None) is not None:
        if not isinstance(params["output_at_counts"], list):
            raise StyxValidationError(f'`output_at_counts` has the wrong type: Received `{type(params.get("output_at_counts", None))}` expected `list[int] | None`')
        for e in params["output_at_counts"]:
            if not isinstance(e, int):
                raise StyxValidationError(f'`output_at_counts` has the wrong type: Received `{type(params.get("output_at_counts", None))}` expected `list[int] | None`')
    if params.get("proc_mask", None) is not None:
        if not isinstance(params["proc_mask"], (pathlib.Path, str)):
            raise StyxValidationError(f'`proc_mask` has the wrong type: Received `{type(params.get("proc_mask", None))}` expected `InputPathType | None`')
    if params.get("act", None) is not None:
        if not isinstance(params["act"], (pathlib.Path, str)):
            raise StyxValidationError(f'`act` has the wrong type: Received `{type(params.get("act", None))}` expected `InputPathType | None`')
    if params.get("fd_scale_gm", False) is None:
        raise StyxValidationError("`fd_scale_gm` must not be None")
    if not isinstance(params["fd_scale_gm"], bool):
        raise StyxValidationError(f'`fd_scale_gm` has the wrong type: Received `{type(params.get("fd_scale_gm", False))}` expected `bool`')
    if params.get("no_dilate_lut", False) is None:
        raise StyxValidationError("`no_dilate_lut` must not be None")
    if not isinstance(params["no_dilate_lut"], bool):
        raise StyxValidationError(f'`no_dilate_lut` has the wrong type: Received `{type(params.get("no_dilate_lut", False))}` expected `bool`')
    if params.get("make_null_lobes", False) is None:
        raise StyxValidationError("`make_null_lobes` must not be None")
    if not isinstance(params["make_null_lobes"], bool):
        raise StyxValidationError(f'`make_null_lobes` has the wrong type: Received `{type(params.get("make_null_lobes", False))}` expected `bool`')
    if params.get("remove_untracked", False) is None:
        raise StyxValidationError("`remove_untracked` must not be None")
    if not isinstance(params["remove_untracked"], bool):
        raise StyxValidationError(f'`remove_untracked` has the wrong type: Received `{type(params.get("remove_untracked", False))}` expected `bool`')
    if params.get("fd_thresh", None) is not None:
        if not isinstance(params["fd_thresh"], (float, int)):
            raise StyxValidationError(f'`fd_thresh` has the wrong type: Received `{type(params.get("fd_thresh", None))}` expected `float | None`')
    if params.get("csv", None) is not None:
        if not isinstance(params["csv"], str):
            raise StyxValidationError(f'`csv` has the wrong type: Received `{type(params.get("csv", None))}` expected `str | None`')
    if params.get("out_mu", None) is not None:
        if not isinstance(params["out_mu"], str):
            raise StyxValidationError(f'`out_mu` has the wrong type: Received `{type(params.get("out_mu", None))}` expected `str | None`')
    if params.get("output_debug", False) is None:
        raise StyxValidationError("`output_debug` must not be None")
    if not isinstance(params["output_debug"], bool):
        raise StyxValidationError(f'`output_debug` has the wrong type: Received `{type(params.get("output_debug", False))}` expected `bool`')
    if params.get("out_selection", None) is not None:
        if not isinstance(params["out_selection"], str):
            raise StyxValidationError(f'`out_selection` has the wrong type: Received `{type(params.get("out_selection", None))}` expected `str | None`')
    if params.get("term_number", None) is not None:
        if not isinstance(params["term_number"], int):
            raise StyxValidationError(f'`term_number` has the wrong type: Received `{type(params.get("term_number", None))}` expected `int | None`')
    if params.get("term_ratio", None) is not None:
        if not isinstance(params["term_ratio"], (float, int)):
            raise StyxValidationError(f'`term_ratio` has the wrong type: Received `{type(params.get("term_ratio", None))}` expected `float | None`')
    if params.get("term_mu", None) is not None:
        if not isinstance(params["term_mu"], (float, int)):
            raise StyxValidationError(f'`term_mu` has the wrong type: Received `{type(params.get("term_mu", None))}` expected `float | None`')
    if params.get("info", False) is None:
        raise StyxValidationError("`info` must not be None")
    if not isinstance(params["info"], bool):
        raise StyxValidationError(f'`info` has the wrong type: Received `{type(params.get("info", False))}` expected `bool`')
    if params.get("quiet", False) is None:
        raise StyxValidationError("`quiet` must not be None")
    if not isinstance(params["quiet"], bool):
        raise StyxValidationError(f'`quiet` has the wrong type: Received `{type(params.get("quiet", False))}` expected `bool`')
    if params.get("debug", False) is None:
        raise StyxValidationError("`debug` must not be None")
    if not isinstance(params["debug"], bool):
        raise StyxValidationError(f'`debug` has the wrong type: Received `{type(params.get("debug", False))}` expected `bool`')
    if params.get("force", False) is None:
        raise StyxValidationError("`force` must not be None")
    if not isinstance(params["force"], bool):
        raise StyxValidationError(f'`force` has the wrong type: Received `{type(params.get("force", False))}` expected `bool`')
    if params.get("nthreads", None) is not None:
        if not isinstance(params["nthreads"], int):
            raise StyxValidationError(f'`nthreads` has the wrong type: Received `{type(params.get("nthreads", None))}` expected `int | None`')
    if params.get("config", None) is not None:
        if not isinstance(params["config"], list):
            raise StyxValidationError(f'`config` has the wrong type: Received `{type(params.get("config", None))}` expected `list[TcksiftConfigParamsDict] | None`')
        for e in params["config"]:
            tcksift_config_validate(e)
    if params.get("help", False) is None:
        raise StyxValidationError("`help` must not be None")
    if not isinstance(params["help"], bool):
        raise StyxValidationError(f'`help` has the wrong type: Received `{type(params.get("help", False))}` expected `bool`')
    if params.get("version", False) is None:
        raise StyxValidationError("`version` must not be None")
    if not isinstance(params["version"], bool):
        raise StyxValidationError(f'`version` has the wrong type: Received `{type(params.get("version", False))}` expected `bool`')
    if params.get("in_tracks", None) is None:
        raise StyxValidationError("`in_tracks` must not be None")
    if not isinstance(params["in_tracks"], (pathlib.Path, str)):
        raise StyxValidationError(f'`in_tracks` has the wrong type: Received `{type(params.get("in_tracks", None))}` expected `InputPathType`')
    if params.get("in_fod", None) is None:
        raise StyxValidationError("`in_fod` must not be None")
    if not isinstance(params["in_fod"], (pathlib.Path, str)):
        raise StyxValidationError(f'`in_fod` has the wrong type: Received `{type(params.get("in_fod", None))}` expected `InputPathType`')
    if params.get("out_tracks", None) is None:
        raise StyxValidationError("`out_tracks` must not be None")
    if not isinstance(params["out_tracks"], str):
        raise StyxValidationError(f'`out_tracks` has the wrong type: Received `{type(params.get("out_tracks", None))}` expected `str`')


def tcksift_cargs(
    params: TcksiftParamsDict,
    execution: Execution,
) -> list[str]:
    """
    Build command-line arguments from parameters.
    
    Args:
        params: The parameters.
        execution: The execution object for resolving input paths.
    Returns:
        Command-line arguments.
    """
    cargs = []
    cargs.append("tcksift")
    if params.get("nofilter", False):
        cargs.append("-nofilter")
    if params.get("output_at_counts", None) is not None:
        cargs.extend([
            "-output_at_counts",
            ",".join(map(str, params.get("output_at_counts", None)))
        ])
    if params.get("proc_mask", None) is not None:
        cargs.extend([
            "-proc_mask",
            execution.input_file(params.get("proc_mask", None))
        ])
    if params.get("act", None) is not None:
        cargs.extend([
            "-act",
            execution.input_file(params.get("act", None))
        ])
    if params.get("fd_scale_gm", False):
        cargs.append("-fd_scale_gm")
    if params.get("no_dilate_lut", False):
        cargs.append("-no_dilate_lut")
    if params.get("make_null_lobes", False):
        cargs.append("-make_null_lobes")
    if params.get("remove_untracked", False):
        cargs.append("-remove_untracked")
    if params.get("fd_thresh", None) is not None:
        cargs.extend([
            "-fd_thresh",
            str(params.get("fd_thresh", None))
        ])
    if params.get("csv", None) is not None:
        cargs.extend([
            "-csv",
            params.get("csv", None)
        ])
    if params.get("out_mu", None) is not None:
        cargs.extend([
            "-out_mu",
            params.get("out_mu", None)
        ])
    if params.get("output_debug", False):
        cargs.append("-output_debug")
    if params.get("out_selection", None) is not None:
        cargs.extend([
            "-out_selection",
            params.get("out_selection", None)
        ])
    if params.get("term_number", None) is not None:
        cargs.extend([
            "-term_number",
            str(params.get("term_number", None))
        ])
    if params.get("term_ratio", None) is not None:
        cargs.extend([
            "-term_ratio",
            str(params.get("term_ratio", None))
        ])
    if params.get("term_mu", None) is not None:
        cargs.extend([
            "-term_mu",
            str(params.get("term_mu", None))
        ])
    if params.get("info", False):
        cargs.append("-info")
    if params.get("quiet", False):
        cargs.append("-quiet")
    if params.get("debug", False):
        cargs.append("-debug")
    if params.get("force", False):
        cargs.append("-force")
    if params.get("nthreads", None) is not None:
        cargs.extend([
            "-nthreads",
            str(params.get("nthreads", None))
        ])
    if params.get("config", None) is not None:
        cargs.extend([a for c in [tcksift_config_cargs(s, execution) for s in params.get("config", None)] for a in c])
    if params.get("help", False):
        cargs.append("-help")
    if params.get("version", False):
        cargs.append("-version")
    cargs.append(execution.input_file(params.get("in_tracks", None)))
    cargs.append(execution.input_file(params.get("in_fod", None)))
    cargs.append(params.get("out_tracks", None))
    return cargs


def tcksift_outputs(
    params: TcksiftParamsDict,
    execution: Execution,
) -> TcksiftOutputs:
    """
    Build outputs object containing output file paths and possibly stdout/stderr.
    
    Args:
        params: The parameters.
        execution: The execution object for resolving input paths.
    Returns:
        Outputs object.
    """
    ret = TcksiftOutputs(
        root=execution.output_file("."),
        out_tracks=execution.output_file(params.get("out_tracks", None)),
        csv_=execution.output_file(params.get("csv", None)) if (params.get("csv") is not None) else None,
        out_mu=execution.output_file(params.get("out_mu", None)) if (params.get("out_mu") is not None) else None,
        out_selection=execution.output_file(params.get("out_selection", None)) if (params.get("out_selection") is not None) else None,
    )
    return ret


def tcksift_execute(
    params: TcksiftParamsDict,
    runner: Runner | None = None,
) -> TcksiftOutputs:
    """
    tcksift
    
    Filter a whole-brain fibre-tracking data set such that the streamline
    densities match the FOD lobe integrals.
    
    
    
    References:
    
    Smith, R. E.; Tournier, J.-D.; Calamante, F. & Connelly, A. SIFT:
    Spherical-deconvolution informed filtering of tractograms. NeuroImage, 2013,
    67, 298-312.
    
    Author: MRTrix3 Developers
    
    URL: https://www.mrtrix.org/
    
    Args:
        params: The parameters.
        runner: Command runner.
    Returns:
        NamedTuple of outputs (described in `TcksiftOutputs`).
    """
    tcksift_validate(params)
    runner = runner or get_global_runner()
    execution = runner.start_execution(TCKSIFT_METADATA)
    params = execution.params(params)
    cargs = tcksift_cargs(params, execution)
    ret = tcksift_outputs(params, execution)
    execution.run(cargs)
    return ret


def tcksift(
    in_tracks: InputPathType,
    in_fod: InputPathType,
    out_tracks: str,
    nofilter: bool = False,
    output_at_counts: list[int] | None = None,
    proc_mask: InputPathType | None = None,
    act: InputPathType | None = None,
    fd_scale_gm: bool = False,
    no_dilate_lut: bool = False,
    make_null_lobes: bool = False,
    remove_untracked: bool = False,
    fd_thresh: float | None = None,
    csv_: str | None = None,
    out_mu: str | None = None,
    output_debug: bool = False,
    out_selection: str | None = None,
    term_number: int | None = None,
    term_ratio: float | None = None,
    term_mu: float | None = None,
    info: bool = False,
    quiet: bool = False,
    debug: bool = False,
    force: bool = False,
    nthreads: int | None = None,
    config: list[TcksiftConfigParamsDict] | None = None,
    help_: bool = False,
    version: bool = False,
    runner: Runner | None = None,
) -> TcksiftOutputs:
    """
    tcksift
    
    Filter a whole-brain fibre-tracking data set such that the streamline
    densities match the FOD lobe integrals.
    
    
    
    References:
    
    Smith, R. E.; Tournier, J.-D.; Calamante, F. & Connelly, A. SIFT:
    Spherical-deconvolution informed filtering of tractograms. NeuroImage, 2013,
    67, 298-312.
    
    Author: MRTrix3 Developers
    
    URL: https://www.mrtrix.org/
    
    Args:
        in_tracks: the input track file.
        in_fod: input image containing the spherical harmonics of the fibre\
            orientation distributions.
        out_tracks: the output filtered tracks file.
        nofilter: do NOT perform track filtering - just construct the model in\
            order to provide output debugging images.
        output_at_counts: output filtered track files (and optionally debugging\
            images if -output_debug is specified) at specific numbers of remaining\
            streamlines; provide as comma-separated list of integers.
        proc_mask: provide an image containing the processing mask weights for\
            the model; image spatial dimensions must match the fixel image.
        act: use an ACT five-tissue-type segmented anatomical image to derive\
            the processing mask.
        fd_scale_gm: provide this option (in conjunction with -act) to\
            heuristically downsize the fibre density estimates based on the\
            presence of GM in the voxel. This can assist in reducing tissue\
            interface effects when using a single-tissue deconvolution algorithm.
        no_dilate_lut: do NOT dilate FOD lobe lookup tables; only map\
            streamlines to FOD lobes if the precise tangent lies within the angular\
            spread of that lobe.
        make_null_lobes: add an additional FOD lobe to each voxel, with zero\
            integral, that covers all directions with zero / negative FOD\
            amplitudes.
        remove_untracked: remove FOD lobes that do not have any streamline\
            density attributed to them; this improves filtering slightly, at the\
            expense of longer computation time (and you can no longer do\
            quantitative comparisons between reconstructions if this is enabled).
        fd_thresh: fibre density threshold; exclude an FOD lobe from filtering\
            processing if its integral is less than this amount (streamlines will\
            still be mapped to it, but it will not contribute to the cost function\
            or the filtering).
        csv_: output statistics of execution per iteration to a .csv file.
        out_mu: output the final value of SIFT proportionality coefficient mu\
            to a text file.
        output_debug: provide various output images for assessing & debugging\
            performance etc.
        out_selection: output a text file containing the binary selection of\
            streamlines.
        term_number: number of streamlines - continue filtering until this\
            number of streamlines remain.
        term_ratio: termination ratio - defined as the ratio between reduction\
            in cost function, and reduction in density of streamlines.\
            Smaller values result in more streamlines being filtered out.
        term_mu: terminate filtering once the SIFT proportionality coefficient\
            reaches a given value.
        info: display information messages.
        quiet: do not display information messages or progress status;\
            alternatively, this can be achieved by setting the MRTRIX_QUIET\
            environment variable to a non-empty string.
        debug: display debugging messages.
        force: force overwrite of output files (caution: using the same file as\
            input and output might cause unexpected behaviour).
        nthreads: use this number of threads in multi-threaded applications\
            (set to 0 to disable multi-threading).
        config: temporarily set the value of an MRtrix config file entry.
        help_: display this information page and exit.
        version: display version information and exit.
        runner: Command runner.
    Returns:
        NamedTuple of outputs (described in `TcksiftOutputs`).
    """
    params = tcksift_params(
        nofilter=nofilter,
        output_at_counts=output_at_counts,
        proc_mask=proc_mask,
        act=act,
        fd_scale_gm=fd_scale_gm,
        no_dilate_lut=no_dilate_lut,
        make_null_lobes=make_null_lobes,
        remove_untracked=remove_untracked,
        fd_thresh=fd_thresh,
        csv_=csv_,
        out_mu=out_mu,
        output_debug=output_debug,
        out_selection=out_selection,
        term_number=term_number,
        term_ratio=term_ratio,
        term_mu=term_mu,
        info=info,
        quiet=quiet,
        debug=debug,
        force=force,
        nthreads=nthreads,
        config=config,
        help_=help_,
        version=version,
        in_tracks=in_tracks,
        in_fod=in_fod,
        out_tracks=out_tracks,
    )
    return tcksift_execute(params, runner)


__all__ = [
    "TCKSIFT_METADATA",
    "TcksiftConfigParamsDict",
    "TcksiftConfigParamsDictTagged",
    "TcksiftOutputs",
    "TcksiftParamsDict",
    "TcksiftParamsDictTagged",
    "tcksift",
    "tcksift_config",
    "tcksift_execute",
    "tcksift_params",
]



# This file was auto generated by Styx.
# Do not edit this file directly.

import typing
import pathlib
from styxdefs import *

FIXELCFESTATS_METADATA = Metadata(
    id="977582ec85abc36abca54062935b4e3d34f0c9f9.boutiques",
    name="fixelcfestats",
    package="mrtrix",
    container_image_tag="mrtrix3/mrtrix3:3.0.4",
)


_FixelcfestatsColumnParamsDictNoTag = typing.TypedDict('_FixelcfestatsColumnParamsDictNoTag', {
    "path": InputPathType,
})
FixelcfestatsColumnParamsDictTagged = typing.TypedDict('FixelcfestatsColumnParamsDictTagged', {
    "@type": typing.Literal["column"],
    "path": InputPathType,
})
FixelcfestatsColumnParamsDict = _FixelcfestatsColumnParamsDictNoTag | FixelcfestatsColumnParamsDictTagged


_FixelcfestatsConfigParamsDictNoTag = typing.TypedDict('_FixelcfestatsConfigParamsDictNoTag', {
    "key": str,
    "value": str,
})
FixelcfestatsConfigParamsDictTagged = typing.TypedDict('FixelcfestatsConfigParamsDictTagged', {
    "@type": typing.Literal["config"],
    "key": str,
    "value": str,
})
FixelcfestatsConfigParamsDict = _FixelcfestatsConfigParamsDictNoTag | FixelcfestatsConfigParamsDictTagged


_FixelcfestatsVariousStringParamsDictNoTag = typing.TypedDict('_FixelcfestatsVariousStringParamsDictNoTag', {
    "obj": str,
})
FixelcfestatsVariousStringParamsDictTagged = typing.TypedDict('FixelcfestatsVariousStringParamsDictTagged', {
    "@type": typing.Literal["VariousString"],
    "obj": str,
})
FixelcfestatsVariousStringParamsDict = _FixelcfestatsVariousStringParamsDictNoTag | FixelcfestatsVariousStringParamsDictTagged


_FixelcfestatsVariousFileParamsDictNoTag = typing.TypedDict('_FixelcfestatsVariousFileParamsDictNoTag', {
    "obj": InputPathType,
})
FixelcfestatsVariousFileParamsDictTagged = typing.TypedDict('FixelcfestatsVariousFileParamsDictTagged', {
    "@type": typing.Literal["VariousFile"],
    "obj": InputPathType,
})
FixelcfestatsVariousFileParamsDict = _FixelcfestatsVariousFileParamsDictNoTag | FixelcfestatsVariousFileParamsDictTagged


_FixelcfestatsParamsDictNoTag = typing.TypedDict('_FixelcfestatsParamsDictNoTag', {
    "mask": typing.NotRequired[InputPathType | None],
    "notest": bool,
    "errors": typing.NotRequired[str | None],
    "exchange_within": typing.NotRequired[InputPathType | None],
    "exchange_whole": typing.NotRequired[InputPathType | None],
    "strong": bool,
    "nshuffles": typing.NotRequired[int | None],
    "permutations": typing.NotRequired[InputPathType | None],
    "nonstationarity": bool,
    "skew_nonstationarity": typing.NotRequired[float | None],
    "nshuffles_nonstationarity": typing.NotRequired[int | None],
    "permutations_nonstationarity": typing.NotRequired[InputPathType | None],
    "cfe_dh": typing.NotRequired[float | None],
    "cfe_e": typing.NotRequired[float | None],
    "cfe_h": typing.NotRequired[float | None],
    "cfe_c": typing.NotRequired[float | None],
    "cfe_legacy": bool,
    "variance": typing.NotRequired[InputPathType | None],
    "ftests": typing.NotRequired[InputPathType | None],
    "fonly": bool,
    "column": typing.NotRequired[list[FixelcfestatsColumnParamsDict] | None],
    "info": bool,
    "quiet": bool,
    "debug": bool,
    "force": bool,
    "nthreads": typing.NotRequired[int | None],
    "config": typing.NotRequired[list[FixelcfestatsConfigParamsDict] | None],
    "help": bool,
    "version": bool,
    "in_fixel_directory": InputPathType,
    "subjects": InputPathType,
    "design": InputPathType,
    "contrast": InputPathType,
    "connectivity": typing.Union[FixelcfestatsVariousStringParamsDictTagged, FixelcfestatsVariousFileParamsDictTagged],
    "out_fixel_directory": str,
})
FixelcfestatsParamsDictTagged = typing.TypedDict('FixelcfestatsParamsDictTagged', {
    "@type": typing.Literal["mrtrix/fixelcfestats"],
    "mask": typing.NotRequired[InputPathType | None],
    "notest": bool,
    "errors": typing.NotRequired[str | None],
    "exchange_within": typing.NotRequired[InputPathType | None],
    "exchange_whole": typing.NotRequired[InputPathType | None],
    "strong": bool,
    "nshuffles": typing.NotRequired[int | None],
    "permutations": typing.NotRequired[InputPathType | None],
    "nonstationarity": bool,
    "skew_nonstationarity": typing.NotRequired[float | None],
    "nshuffles_nonstationarity": typing.NotRequired[int | None],
    "permutations_nonstationarity": typing.NotRequired[InputPathType | None],
    "cfe_dh": typing.NotRequired[float | None],
    "cfe_e": typing.NotRequired[float | None],
    "cfe_h": typing.NotRequired[float | None],
    "cfe_c": typing.NotRequired[float | None],
    "cfe_legacy": bool,
    "variance": typing.NotRequired[InputPathType | None],
    "ftests": typing.NotRequired[InputPathType | None],
    "fonly": bool,
    "column": typing.NotRequired[list[FixelcfestatsColumnParamsDict] | None],
    "info": bool,
    "quiet": bool,
    "debug": bool,
    "force": bool,
    "nthreads": typing.NotRequired[int | None],
    "config": typing.NotRequired[list[FixelcfestatsConfigParamsDict] | None],
    "help": bool,
    "version": bool,
    "in_fixel_directory": InputPathType,
    "subjects": InputPathType,
    "design": InputPathType,
    "contrast": InputPathType,
    "connectivity": typing.Union[FixelcfestatsVariousStringParamsDictTagged, FixelcfestatsVariousFileParamsDictTagged],
    "out_fixel_directory": str,
})
FixelcfestatsParamsDict = _FixelcfestatsParamsDictNoTag | FixelcfestatsParamsDictTagged


def fixelcfestats_connectivity_cargs_dyn_fn(
    t: str,
) -> typing.Any:
    """
    Get build cargs function by command type.
    
    Args:
        t: Command type.
    Returns:
        Build cargs function.
    """
    return {
        "VariousString": fixelcfestats_various_string_cargs,
        "VariousFile": fixelcfestats_various_file_cargs,
    }.get(t)


def fixelcfestats_connectivity_validate_dyn_fn(
    t: str,
) -> typing.Any:
    """
    Get validate params function by command type.
    
    Args:
        t: Command type.
    Returns:
        Validate params function.
    """
    return {
        "VariousString": fixelcfestats_various_string_validate,
        "VariousFile": fixelcfestats_various_file_validate,
    }.get(t)


def fixelcfestats_column(
    path: InputPathType,
) -> FixelcfestatsColumnParamsDictTagged:
    """
    Build parameters.
    
    Args:
        path: add a column to the design matrix corresponding to subject\
            fixel-wise values (note that the contrast matrix must include an\
            additional column for each use of this option); the text file provided\
            via this option should contain a file name for each subject.
    Returns:
        Parameter dictionary
    """
    params = {
        "@type": "column",
        "path": path,
    }
    return params


def fixelcfestats_column_validate(
    params: typing.Any,
) -> None:
    """
    Validate parameters. Throws an error if `params` is not a valid
    `FixelcfestatsColumnParamsDict` object.
    
    Args:
        params: The parameters object to validate.
    """
    if params is None or not isinstance(params, dict):
        raise StyxValidationError(f'Params object has the wrong type \'{type(params)}\'')
    if params.get("path", None) is None:
        raise StyxValidationError("`path` must not be None")
    if not isinstance(params["path"], (pathlib.Path, str)):
        raise StyxValidationError(f'`path` has the wrong type: Received `{type(params.get("path", None))}` expected `InputPathType`')


def fixelcfestats_column_cargs(
    params: FixelcfestatsColumnParamsDict,
    execution: Execution,
) -> list[str]:
    """
    Build command-line arguments from parameters.
    
    Args:
        params: The parameters.
        execution: The execution object for resolving input paths.
    Returns:
        Command-line arguments.
    """
    cargs = []
    cargs.append("-column")
    cargs.append(execution.input_file(params.get("path", None)))
    return cargs


def fixelcfestats_config(
    key: str,
    value: str,
) -> FixelcfestatsConfigParamsDictTagged:
    """
    Build parameters.
    
    Args:
        key: temporarily set the value of an MRtrix config file entry.
        value: temporarily set the value of an MRtrix config file entry.
    Returns:
        Parameter dictionary
    """
    params = {
        "@type": "config",
        "key": key,
        "value": value,
    }
    return params


def fixelcfestats_config_validate(
    params: typing.Any,
) -> None:
    """
    Validate parameters. Throws an error if `params` is not a valid
    `FixelcfestatsConfigParamsDict` object.
    
    Args:
        params: The parameters object to validate.
    """
    if params is None or not isinstance(params, dict):
        raise StyxValidationError(f'Params object has the wrong type \'{type(params)}\'')
    if params.get("key", None) is None:
        raise StyxValidationError("`key` must not be None")
    if not isinstance(params["key"], str):
        raise StyxValidationError(f'`key` has the wrong type: Received `{type(params.get("key", None))}` expected `str`')
    if params.get("value", None) is None:
        raise StyxValidationError("`value` must not be None")
    if not isinstance(params["value"], str):
        raise StyxValidationError(f'`value` has the wrong type: Received `{type(params.get("value", None))}` expected `str`')


def fixelcfestats_config_cargs(
    params: FixelcfestatsConfigParamsDict,
    execution: Execution,
) -> list[str]:
    """
    Build command-line arguments from parameters.
    
    Args:
        params: The parameters.
        execution: The execution object for resolving input paths.
    Returns:
        Command-line arguments.
    """
    cargs = []
    cargs.append("-config")
    cargs.append(params.get("key", None))
    cargs.append(params.get("value", None))
    return cargs


def fixelcfestats_various_string(
    obj: str,
) -> FixelcfestatsVariousStringParamsDictTagged:
    """
    Build parameters.
    
    Args:
        obj: String object.
    Returns:
        Parameter dictionary
    """
    params = {
        "@type": "VariousString",
        "obj": obj,
    }
    return params


def fixelcfestats_various_string_validate(
    params: typing.Any,
) -> None:
    """
    Validate parameters. Throws an error if `params` is not a valid
    `FixelcfestatsVariousStringParamsDict` object.
    
    Args:
        params: The parameters object to validate.
    """
    if params is None or not isinstance(params, dict):
        raise StyxValidationError(f'Params object has the wrong type \'{type(params)}\'')
    if params.get("obj", None) is None:
        raise StyxValidationError("`obj` must not be None")
    if not isinstance(params["obj"], str):
        raise StyxValidationError(f'`obj` has the wrong type: Received `{type(params.get("obj", None))}` expected `str`')


def fixelcfestats_various_string_cargs(
    params: FixelcfestatsVariousStringParamsDict,
    execution: Execution,
) -> list[str]:
    """
    Build command-line arguments from parameters.
    
    Args:
        params: The parameters.
        execution: The execution object for resolving input paths.
    Returns:
        Command-line arguments.
    """
    cargs = []
    cargs.append(params.get("obj", None))
    return cargs


def fixelcfestats_various_file(
    obj: InputPathType,
) -> FixelcfestatsVariousFileParamsDictTagged:
    """
    Build parameters.
    
    Args:
        obj: File object.
    Returns:
        Parameter dictionary
    """
    params = {
        "@type": "VariousFile",
        "obj": obj,
    }
    return params


def fixelcfestats_various_file_validate(
    params: typing.Any,
) -> None:
    """
    Validate parameters. Throws an error if `params` is not a valid
    `FixelcfestatsVariousFileParamsDict` object.
    
    Args:
        params: The parameters object to validate.
    """
    if params is None or not isinstance(params, dict):
        raise StyxValidationError(f'Params object has the wrong type \'{type(params)}\'')
    if params.get("obj", None) is None:
        raise StyxValidationError("`obj` must not be None")
    if not isinstance(params["obj"], (pathlib.Path, str)):
        raise StyxValidationError(f'`obj` has the wrong type: Received `{type(params.get("obj", None))}` expected `InputPathType`')


def fixelcfestats_various_file_cargs(
    params: FixelcfestatsVariousFileParamsDict,
    execution: Execution,
) -> list[str]:
    """
    Build command-line arguments from parameters.
    
    Args:
        params: The parameters.
        execution: The execution object for resolving input paths.
    Returns:
        Command-line arguments.
    """
    cargs = []
    cargs.append(execution.input_file(params.get("obj", None)))
    return cargs


class FixelcfestatsOutputs(typing.NamedTuple):
    """
    Output object returned when calling `FixelcfestatsParamsDict(...)`.
    """
    root: OutputPathType
    """Output root folder. This is the root folder for all outputs."""


def fixelcfestats_params(
    in_fixel_directory: InputPathType,
    subjects: InputPathType,
    design: InputPathType,
    contrast: InputPathType,
    connectivity: typing.Union[FixelcfestatsVariousStringParamsDictTagged, FixelcfestatsVariousFileParamsDictTagged],
    out_fixel_directory: str,
    mask: InputPathType | None = None,
    notest: bool = False,
    errors: str | None = None,
    exchange_within: InputPathType | None = None,
    exchange_whole: InputPathType | None = None,
    strong: bool = False,
    nshuffles: int | None = None,
    permutations: InputPathType | None = None,
    nonstationarity: bool = False,
    skew_nonstationarity: float | None = None,
    nshuffles_nonstationarity: int | None = None,
    permutations_nonstationarity: InputPathType | None = None,
    cfe_dh: float | None = None,
    cfe_e: float | None = None,
    cfe_h: float | None = None,
    cfe_c: float | None = None,
    cfe_legacy: bool = False,
    variance: InputPathType | None = None,
    ftests: InputPathType | None = None,
    fonly: bool = False,
    column: list[FixelcfestatsColumnParamsDict] | None = None,
    info: bool = False,
    quiet: bool = False,
    debug: bool = False,
    force: bool = False,
    nthreads: int | None = None,
    config: list[FixelcfestatsConfigParamsDict] | None = None,
    help_: bool = False,
    version: bool = False,
) -> FixelcfestatsParamsDictTagged:
    """
    Build parameters.
    
    Args:
        in_fixel_directory: the fixel directory containing the data files for\
            each subject (after obtaining fixel correspondence.
        subjects: a text file listing the subject identifiers (one per line).\
            This should correspond with the filenames in the fixel directory\
            (including the file extension), and be listed in the same order as the\
            rows of the design matrix.
        design: the design matrix.
        contrast: the contrast matrix, specified as rows of weights.
        connectivity: the fixel-fixel connectivity matrix.
        out_fixel_directory: the output directory where results will be saved.\
            Will be created if it does not exist.
        mask: provide a fixel data file containing a mask of those fixels to be\
            used during processing.
        notest: don't perform statistical inference; only output population\
            statistics (effect size, stdev etc).
        errors: specify nature of errors for shuffling; options are:\
            ee,ise,both (default: ee).
        exchange_within: specify blocks of observations within each of which\
            data may undergo restricted exchange.
        exchange_whole: specify blocks of observations that may be exchanged\
            with one another (for independent and symmetric errors, sign-flipping\
            will occur block-wise).
        strong: use strong familywise error control across multiple hypotheses.
        nshuffles: the number of shuffles (default: 5000).
        permutations: manually define the permutations (relabelling). The input\
            should be a text file defining a m x n matrix, where each relabelling\
            is defined as a column vector of size m, and the number of columns, n,\
            defines the number of permutations. Can be generated with the\
            palm_quickperms function in PALM\
            (http://fsl.fmrib.ox.ac.uk/fsl/fslwiki/PALM). Overrides the -nshuffles\
            option.
        nonstationarity: perform non-stationarity correction.
        skew_nonstationarity: specify the skew parameter for empirical\
            statistic calculation (default for this command is 1).
        nshuffles_nonstationarity: the number of shuffles to use when\
            precomputing the empirical statistic image for non-stationarity\
            correction (default: 5000).
        permutations_nonstationarity: manually define the permutations\
            (relabelling) for computing the emprical statistics for\
            non-stationarity correction. The input should be a text file defining a\
            m x n matrix, where each relabelling is defined as a column vector of\
            size m, and the number of columns, n, defines the number of\
            permutations. Can be generated with the palm_quickperms function in\
            PALM (http://fsl.fmrib.ox.ac.uk/fsl/fslwiki/PALM) Overrides the\
            -nshuffles_nonstationarity option.
        cfe_dh: the height increment used in the cfe integration (default: 0.1).
        cfe_e: cfe extent exponent (default: 2).
        cfe_h: cfe height exponent (default: 3).
        cfe_c: cfe connectivity exponent (default: 0.5).
        cfe_legacy: use the legacy (non-normalised) form of the cfe equation.
        variance: define variance groups for the G-statistic; measurements for\
            which the expected variance is equivalent should contain the same index.
        ftests: perform F-tests; input text file should contain, for each\
            F-test, a row containing ones and zeros, where ones indicate the rows\
            of the contrast matrix to be included in the F-test.
        fonly: only assess F-tests; do not perform statistical inference on\
            entries in the contrast matrix.
        column: add a column to the design matrix corresponding to subject\
            fixel-wise values (note that the contrast matrix must include an\
            additional column for each use of this option); the text file provided\
            via this option should contain a file name for each subject.
        info: display information messages.
        quiet: do not display information messages or progress status;\
            alternatively, this can be achieved by setting the MRTRIX_QUIET\
            environment variable to a non-empty string.
        debug: display debugging messages.
        force: force overwrite of output files (caution: using the same file as\
            input and output might cause unexpected behaviour).
        nthreads: use this number of threads in multi-threaded applications\
            (set to 0 to disable multi-threading).
        config: temporarily set the value of an MRtrix config file entry.
        help_: display this information page and exit.
        version: display version information and exit.
    Returns:
        Parameter dictionary
    """
    params = {
        "@type": "mrtrix/fixelcfestats",
        "notest": notest,
        "strong": strong,
        "nonstationarity": nonstationarity,
        "cfe_legacy": cfe_legacy,
        "fonly": fonly,
        "info": info,
        "quiet": quiet,
        "debug": debug,
        "force": force,
        "help": help_,
        "version": version,
        "in_fixel_directory": in_fixel_directory,
        "subjects": subjects,
        "design": design,
        "contrast": contrast,
        "connectivity": connectivity,
        "out_fixel_directory": out_fixel_directory,
    }
    if mask is not None:
        params["mask"] = mask
    if errors is not None:
        params["errors"] = errors
    if exchange_within is not None:
        params["exchange_within"] = exchange_within
    if exchange_whole is not None:
        params["exchange_whole"] = exchange_whole
    if nshuffles is not None:
        params["nshuffles"] = nshuffles
    if permutations is not None:
        params["permutations"] = permutations
    if skew_nonstationarity is not None:
        params["skew_nonstationarity"] = skew_nonstationarity
    if nshuffles_nonstationarity is not None:
        params["nshuffles_nonstationarity"] = nshuffles_nonstationarity
    if permutations_nonstationarity is not None:
        params["permutations_nonstationarity"] = permutations_nonstationarity
    if cfe_dh is not None:
        params["cfe_dh"] = cfe_dh
    if cfe_e is not None:
        params["cfe_e"] = cfe_e
    if cfe_h is not None:
        params["cfe_h"] = cfe_h
    if cfe_c is not None:
        params["cfe_c"] = cfe_c
    if variance is not None:
        params["variance"] = variance
    if ftests is not None:
        params["ftests"] = ftests
    if column is not None:
        params["column"] = column
    if nthreads is not None:
        params["nthreads"] = nthreads
    if config is not None:
        params["config"] = config
    return params


def fixelcfestats_validate(
    params: typing.Any,
) -> None:
    """
    Validate parameters. Throws an error if `params` is not a valid
    `FixelcfestatsParamsDict` object.
    
    Args:
        params: The parameters object to validate.
    """
    if params is None or not isinstance(params, dict):
        raise StyxValidationError(f'Params object has the wrong type \'{type(params)}\'')
    if params.get("mask", None) is not None:
        if not isinstance(params["mask"], (pathlib.Path, str)):
            raise StyxValidationError(f'`mask` has the wrong type: Received `{type(params.get("mask", None))}` expected `InputPathType | None`')
    if params.get("notest", False) is None:
        raise StyxValidationError("`notest` must not be None")
    if not isinstance(params["notest"], bool):
        raise StyxValidationError(f'`notest` has the wrong type: Received `{type(params.get("notest", False))}` expected `bool`')
    if params.get("errors", None) is not None:
        if not isinstance(params["errors"], str):
            raise StyxValidationError(f'`errors` has the wrong type: Received `{type(params.get("errors", None))}` expected `str | None`')
    if params.get("exchange_within", None) is not None:
        if not isinstance(params["exchange_within"], (pathlib.Path, str)):
            raise StyxValidationError(f'`exchange_within` has the wrong type: Received `{type(params.get("exchange_within", None))}` expected `InputPathType | None`')
    if params.get("exchange_whole", None) is not None:
        if not isinstance(params["exchange_whole"], (pathlib.Path, str)):
            raise StyxValidationError(f'`exchange_whole` has the wrong type: Received `{type(params.get("exchange_whole", None))}` expected `InputPathType | None`')
    if params.get("strong", False) is None:
        raise StyxValidationError("`strong` must not be None")
    if not isinstance(params["strong"], bool):
        raise StyxValidationError(f'`strong` has the wrong type: Received `{type(params.get("strong", False))}` expected `bool`')
    if params.get("nshuffles", None) is not None:
        if not isinstance(params["nshuffles"], int):
            raise StyxValidationError(f'`nshuffles` has the wrong type: Received `{type(params.get("nshuffles", None))}` expected `int | None`')
    if params.get("permutations", None) is not None:
        if not isinstance(params["permutations"], (pathlib.Path, str)):
            raise StyxValidationError(f'`permutations` has the wrong type: Received `{type(params.get("permutations", None))}` expected `InputPathType | None`')
    if params.get("nonstationarity", False) is None:
        raise StyxValidationError("`nonstationarity` must not be None")
    if not isinstance(params["nonstationarity"], bool):
        raise StyxValidationError(f'`nonstationarity` has the wrong type: Received `{type(params.get("nonstationarity", False))}` expected `bool`')
    if params.get("skew_nonstationarity", None) is not None:
        if not isinstance(params["skew_nonstationarity"], (float, int)):
            raise StyxValidationError(f'`skew_nonstationarity` has the wrong type: Received `{type(params.get("skew_nonstationarity", None))}` expected `float | None`')
    if params.get("nshuffles_nonstationarity", None) is not None:
        if not isinstance(params["nshuffles_nonstationarity"], int):
            raise StyxValidationError(f'`nshuffles_nonstationarity` has the wrong type: Received `{type(params.get("nshuffles_nonstationarity", None))}` expected `int | None`')
    if params.get("permutations_nonstationarity", None) is not None:
        if not isinstance(params["permutations_nonstationarity"], (pathlib.Path, str)):
            raise StyxValidationError(f'`permutations_nonstationarity` has the wrong type: Received `{type(params.get("permutations_nonstationarity", None))}` expected `InputPathType | None`')
    if params.get("cfe_dh", None) is not None:
        if not isinstance(params["cfe_dh"], (float, int)):
            raise StyxValidationError(f'`cfe_dh` has the wrong type: Received `{type(params.get("cfe_dh", None))}` expected `float | None`')
    if params.get("cfe_e", None) is not None:
        if not isinstance(params["cfe_e"], (float, int)):
            raise StyxValidationError(f'`cfe_e` has the wrong type: Received `{type(params.get("cfe_e", None))}` expected `float | None`')
    if params.get("cfe_h", None) is not None:
        if not isinstance(params["cfe_h"], (float, int)):
            raise StyxValidationError(f'`cfe_h` has the wrong type: Received `{type(params.get("cfe_h", None))}` expected `float | None`')
    if params.get("cfe_c", None) is not None:
        if not isinstance(params["cfe_c"], (float, int)):
            raise StyxValidationError(f'`cfe_c` has the wrong type: Received `{type(params.get("cfe_c", None))}` expected `float | None`')
    if params.get("cfe_legacy", False) is None:
        raise StyxValidationError("`cfe_legacy` must not be None")
    if not isinstance(params["cfe_legacy"], bool):
        raise StyxValidationError(f'`cfe_legacy` has the wrong type: Received `{type(params.get("cfe_legacy", False))}` expected `bool`')
    if params.get("variance", None) is not None:
        if not isinstance(params["variance"], (pathlib.Path, str)):
            raise StyxValidationError(f'`variance` has the wrong type: Received `{type(params.get("variance", None))}` expected `InputPathType | None`')
    if params.get("ftests", None) is not None:
        if not isinstance(params["ftests"], (pathlib.Path, str)):
            raise StyxValidationError(f'`ftests` has the wrong type: Received `{type(params.get("ftests", None))}` expected `InputPathType | None`')
    if params.get("fonly", False) is None:
        raise StyxValidationError("`fonly` must not be None")
    if not isinstance(params["fonly"], bool):
        raise StyxValidationError(f'`fonly` has the wrong type: Received `{type(params.get("fonly", False))}` expected `bool`')
    if params.get("column", None) is not None:
        if not isinstance(params["column"], list):
            raise StyxValidationError(f'`column` has the wrong type: Received `{type(params.get("column", None))}` expected `list[FixelcfestatsColumnParamsDict] | None`')
        for e in params["column"]:
            fixelcfestats_column_validate(e)
    if params.get("info", False) is None:
        raise StyxValidationError("`info` must not be None")
    if not isinstance(params["info"], bool):
        raise StyxValidationError(f'`info` has the wrong type: Received `{type(params.get("info", False))}` expected `bool`')
    if params.get("quiet", False) is None:
        raise StyxValidationError("`quiet` must not be None")
    if not isinstance(params["quiet"], bool):
        raise StyxValidationError(f'`quiet` has the wrong type: Received `{type(params.get("quiet", False))}` expected `bool`')
    if params.get("debug", False) is None:
        raise StyxValidationError("`debug` must not be None")
    if not isinstance(params["debug"], bool):
        raise StyxValidationError(f'`debug` has the wrong type: Received `{type(params.get("debug", False))}` expected `bool`')
    if params.get("force", False) is None:
        raise StyxValidationError("`force` must not be None")
    if not isinstance(params["force"], bool):
        raise StyxValidationError(f'`force` has the wrong type: Received `{type(params.get("force", False))}` expected `bool`')
    if params.get("nthreads", None) is not None:
        if not isinstance(params["nthreads"], int):
            raise StyxValidationError(f'`nthreads` has the wrong type: Received `{type(params.get("nthreads", None))}` expected `int | None`')
    if params.get("config", None) is not None:
        if not isinstance(params["config"], list):
            raise StyxValidationError(f'`config` has the wrong type: Received `{type(params.get("config", None))}` expected `list[FixelcfestatsConfigParamsDict] | None`')
        for e in params["config"]:
            fixelcfestats_config_validate(e)
    if params.get("help", False) is None:
        raise StyxValidationError("`help` must not be None")
    if not isinstance(params["help"], bool):
        raise StyxValidationError(f'`help` has the wrong type: Received `{type(params.get("help", False))}` expected `bool`')
    if params.get("version", False) is None:
        raise StyxValidationError("`version` must not be None")
    if not isinstance(params["version"], bool):
        raise StyxValidationError(f'`version` has the wrong type: Received `{type(params.get("version", False))}` expected `bool`')
    if params.get("in_fixel_directory", None) is None:
        raise StyxValidationError("`in_fixel_directory` must not be None")
    if not isinstance(params["in_fixel_directory"], (pathlib.Path, str)):
        raise StyxValidationError(f'`in_fixel_directory` has the wrong type: Received `{type(params.get("in_fixel_directory", None))}` expected `InputPathType`')
    if params.get("subjects", None) is None:
        raise StyxValidationError("`subjects` must not be None")
    if not isinstance(params["subjects"], (pathlib.Path, str)):
        raise StyxValidationError(f'`subjects` has the wrong type: Received `{type(params.get("subjects", None))}` expected `InputPathType`')
    if params.get("design", None) is None:
        raise StyxValidationError("`design` must not be None")
    if not isinstance(params["design"], (pathlib.Path, str)):
        raise StyxValidationError(f'`design` has the wrong type: Received `{type(params.get("design", None))}` expected `InputPathType`')
    if params.get("contrast", None) is None:
        raise StyxValidationError("`contrast` must not be None")
    if not isinstance(params["contrast"], (pathlib.Path, str)):
        raise StyxValidationError(f'`contrast` has the wrong type: Received `{type(params.get("contrast", None))}` expected `InputPathType`')
    if params.get("connectivity", None) is None:
        raise StyxValidationError("`connectivity` must not be None")
    if not isinstance(params["connectivity"], dict):
        raise StyxValidationError(f'Params object has the wrong type \'{type(params["connectivity"])}\'')
    if "@type" not in params["connectivity"]:
        raise StyxValidationError("Params object is missing `@type`")
    if params["connectivity"]["@type"] not in ["VariousString", "VariousFile"]:
        raise StyxValidationError("Parameter `connectivity`s `@type` must be one of [\"VariousString\", \"VariousFile\"]")
    fixelcfestats_connectivity_validate_dyn_fn(params["connectivity"]["@type"])(params["connectivity"])
    if params.get("out_fixel_directory", None) is None:
        raise StyxValidationError("`out_fixel_directory` must not be None")
    if not isinstance(params["out_fixel_directory"], str):
        raise StyxValidationError(f'`out_fixel_directory` has the wrong type: Received `{type(params.get("out_fixel_directory", None))}` expected `str`')


def fixelcfestats_cargs(
    params: FixelcfestatsParamsDict,
    execution: Execution,
) -> list[str]:
    """
    Build command-line arguments from parameters.
    
    Args:
        params: The parameters.
        execution: The execution object for resolving input paths.
    Returns:
        Command-line arguments.
    """
    cargs = []
    cargs.append("fixelcfestats")
    if params.get("mask", None) is not None:
        cargs.extend([
            "-mask",
            execution.input_file(params.get("mask", None))
        ])
    if params.get("notest", False):
        cargs.append("-notest")
    if params.get("errors", None) is not None:
        cargs.extend([
            "-errors",
            params.get("errors", None)
        ])
    if params.get("exchange_within", None) is not None:
        cargs.extend([
            "-exchange_within",
            execution.input_file(params.get("exchange_within", None))
        ])
    if params.get("exchange_whole", None) is not None:
        cargs.extend([
            "-exchange_whole",
            execution.input_file(params.get("exchange_whole", None))
        ])
    if params.get("strong", False):
        cargs.append("-strong")
    if params.get("nshuffles", None) is not None:
        cargs.extend([
            "-nshuffles",
            str(params.get("nshuffles", None))
        ])
    if params.get("permutations", None) is not None:
        cargs.extend([
            "-permutations",
            execution.input_file(params.get("permutations", None))
        ])
    if params.get("nonstationarity", False):
        cargs.append("-nonstationarity")
    if params.get("skew_nonstationarity", None) is not None:
        cargs.extend([
            "-skew_nonstationarity",
            str(params.get("skew_nonstationarity", None))
        ])
    if params.get("nshuffles_nonstationarity", None) is not None:
        cargs.extend([
            "-nshuffles_nonstationarity",
            str(params.get("nshuffles_nonstationarity", None))
        ])
    if params.get("permutations_nonstationarity", None) is not None:
        cargs.extend([
            "-permutations_nonstationarity",
            execution.input_file(params.get("permutations_nonstationarity", None))
        ])
    if params.get("cfe_dh", None) is not None:
        cargs.extend([
            "-cfe_dh",
            str(params.get("cfe_dh", None))
        ])
    if params.get("cfe_e", None) is not None:
        cargs.extend([
            "-cfe_e",
            str(params.get("cfe_e", None))
        ])
    if params.get("cfe_h", None) is not None:
        cargs.extend([
            "-cfe_h",
            str(params.get("cfe_h", None))
        ])
    if params.get("cfe_c", None) is not None:
        cargs.extend([
            "-cfe_c",
            str(params.get("cfe_c", None))
        ])
    if params.get("cfe_legacy", False):
        cargs.append("-cfe_legacy")
    if params.get("variance", None) is not None:
        cargs.extend([
            "-variance",
            execution.input_file(params.get("variance", None))
        ])
    if params.get("ftests", None) is not None:
        cargs.extend([
            "-ftests",
            execution.input_file(params.get("ftests", None))
        ])
    if params.get("fonly", False):
        cargs.append("-fonly")
    if params.get("column", None) is not None:
        cargs.extend([a for c in [fixelcfestats_column_cargs(s, execution) for s in params.get("column", None)] for a in c])
    if params.get("info", False):
        cargs.append("-info")
    if params.get("quiet", False):
        cargs.append("-quiet")
    if params.get("debug", False):
        cargs.append("-debug")
    if params.get("force", False):
        cargs.append("-force")
    if params.get("nthreads", None) is not None:
        cargs.extend([
            "-nthreads",
            str(params.get("nthreads", None))
        ])
    if params.get("config", None) is not None:
        cargs.extend([a for c in [fixelcfestats_config_cargs(s, execution) for s in params.get("config", None)] for a in c])
    if params.get("help", False):
        cargs.append("-help")
    if params.get("version", False):
        cargs.append("-version")
    cargs.append(execution.input_file(params.get("in_fixel_directory", None)))
    cargs.append(execution.input_file(params.get("subjects", None)))
    cargs.append(execution.input_file(params.get("design", None)))
    cargs.append(execution.input_file(params.get("contrast", None)))
    cargs.extend(fixelcfestats_connectivity_cargs_dyn_fn(params.get("connectivity", None)["@type"])(params.get("connectivity", None), execution))
    cargs.append(params.get("out_fixel_directory", None))
    return cargs


def fixelcfestats_outputs(
    params: FixelcfestatsParamsDict,
    execution: Execution,
) -> FixelcfestatsOutputs:
    """
    Build outputs object containing output file paths and possibly stdout/stderr.
    
    Args:
        params: The parameters.
        execution: The execution object for resolving input paths.
    Returns:
        Outputs object.
    """
    ret = FixelcfestatsOutputs(
        root=execution.output_file("."),
    )
    return ret


def fixelcfestats_execute(
    params: FixelcfestatsParamsDict,
    runner: Runner | None = None,
) -> FixelcfestatsOutputs:
    """
    fixelcfestats
    
    Fixel-based analysis using connectivity-based fixel enhancement and
    non-parametric permutation testing.
    
    Unlike previous versions of this command, where a whole-brain tractogram
    file would be provided as input in order to generate the fixel-fixel
    connectivity matrix and smooth fixel data, this version expects to be
    provided with the directory path to a pre-calculated fixel-fixel
    connectivity matrix (likely generated using the MRtrix3 command
    fixelconnectivity), and for the input fixel data to have already been
    smoothed (likely using the MRtrix3 command fixelfilter).
    
    Note that if the -mask option is used, the output fixel directory will still
    contain the same set of fixels as that present in the input fixel template,
    in order to retain fixel correspondence. However a consequence of this is
    that all fixels in the template will be initialy visible when the output
    fixel directory is loaded in mrview. Those fixels outside the processing
    mask will immediately disappear from view as soon as any data-file-based
    fixel colouring or thresholding is applied.
    
    In some software packages, a column of ones is automatically added to the
    GLM design matrix; the purpose of this column is to estimate the "global
    intercept", which is the predicted value of the observed variable if all
    explanatory variables were to be zero. However there are rare situations
    where including such a column would not be appropriate for a particular
    experimental design. Hence, in MRtrix3 statistical inference commands, it is
    up to the user to determine whether or not this column of ones should be
    included in their design matrix, and add it explicitly if necessary. The
    contrast matrix must also reflect the presence of this additional column.
    
    References:
    
    Raffelt, D.; Smith, RE.; Ridgway, GR.; Tournier, JD.; Vaughan, DN.; Rose,
    S.; Henderson, R.; Connelly, A. Connectivity-based fixel enhancement:
    Whole-brain statistical analysis of diffusion MRI measures in the presence
    of crossing fibres.Neuroimage, 2015, 15(117):40-55
    
    * If not using the -cfe_legacy option:
    Smith, RE.; Dimond, D; Vaughan, D.; Parker, D.; Dhollander, T.; Jackson, G.;
    Connelly, A. Intrinsic non-stationarity correction for Fixel-Based Analysis.
    In Proc OHBM 2019 M789
    
    * If using the -nonstationary option:
    Salimi-Khorshidi, G. Smith, S.M. Nichols, T.E. Adjusting the effect of
    nonstationarity in cluster-based and TFCE inference. NeuroImage, 2011,
    54(3), 2006-19.
    
    Author: MRTrix3 Developers
    
    URL: https://www.mrtrix.org/
    
    Args:
        params: The parameters.
        runner: Command runner.
    Returns:
        NamedTuple of outputs (described in `FixelcfestatsOutputs`).
    """
    fixelcfestats_validate(params)
    runner = runner or get_global_runner()
    execution = runner.start_execution(FIXELCFESTATS_METADATA)
    params = execution.params(params)
    cargs = fixelcfestats_cargs(params, execution)
    ret = fixelcfestats_outputs(params, execution)
    execution.run(cargs)
    return ret


def fixelcfestats(
    in_fixel_directory: InputPathType,
    subjects: InputPathType,
    design: InputPathType,
    contrast: InputPathType,
    connectivity: typing.Union[FixelcfestatsVariousStringParamsDictTagged, FixelcfestatsVariousFileParamsDictTagged],
    out_fixel_directory: str,
    mask: InputPathType | None = None,
    notest: bool = False,
    errors: str | None = None,
    exchange_within: InputPathType | None = None,
    exchange_whole: InputPathType | None = None,
    strong: bool = False,
    nshuffles: int | None = None,
    permutations: InputPathType | None = None,
    nonstationarity: bool = False,
    skew_nonstationarity: float | None = None,
    nshuffles_nonstationarity: int | None = None,
    permutations_nonstationarity: InputPathType | None = None,
    cfe_dh: float | None = None,
    cfe_e: float | None = None,
    cfe_h: float | None = None,
    cfe_c: float | None = None,
    cfe_legacy: bool = False,
    variance: InputPathType | None = None,
    ftests: InputPathType | None = None,
    fonly: bool = False,
    column: list[FixelcfestatsColumnParamsDict] | None = None,
    info: bool = False,
    quiet: bool = False,
    debug: bool = False,
    force: bool = False,
    nthreads: int | None = None,
    config: list[FixelcfestatsConfigParamsDict] | None = None,
    help_: bool = False,
    version: bool = False,
    runner: Runner | None = None,
) -> FixelcfestatsOutputs:
    """
    fixelcfestats
    
    Fixel-based analysis using connectivity-based fixel enhancement and
    non-parametric permutation testing.
    
    Unlike previous versions of this command, where a whole-brain tractogram
    file would be provided as input in order to generate the fixel-fixel
    connectivity matrix and smooth fixel data, this version expects to be
    provided with the directory path to a pre-calculated fixel-fixel
    connectivity matrix (likely generated using the MRtrix3 command
    fixelconnectivity), and for the input fixel data to have already been
    smoothed (likely using the MRtrix3 command fixelfilter).
    
    Note that if the -mask option is used, the output fixel directory will still
    contain the same set of fixels as that present in the input fixel template,
    in order to retain fixel correspondence. However a consequence of this is
    that all fixels in the template will be initialy visible when the output
    fixel directory is loaded in mrview. Those fixels outside the processing
    mask will immediately disappear from view as soon as any data-file-based
    fixel colouring or thresholding is applied.
    
    In some software packages, a column of ones is automatically added to the
    GLM design matrix; the purpose of this column is to estimate the "global
    intercept", which is the predicted value of the observed variable if all
    explanatory variables were to be zero. However there are rare situations
    where including such a column would not be appropriate for a particular
    experimental design. Hence, in MRtrix3 statistical inference commands, it is
    up to the user to determine whether or not this column of ones should be
    included in their design matrix, and add it explicitly if necessary. The
    contrast matrix must also reflect the presence of this additional column.
    
    References:
    
    Raffelt, D.; Smith, RE.; Ridgway, GR.; Tournier, JD.; Vaughan, DN.; Rose,
    S.; Henderson, R.; Connelly, A. Connectivity-based fixel enhancement:
    Whole-brain statistical analysis of diffusion MRI measures in the presence
    of crossing fibres.Neuroimage, 2015, 15(117):40-55
    
    * If not using the -cfe_legacy option:
    Smith, RE.; Dimond, D; Vaughan, D.; Parker, D.; Dhollander, T.; Jackson, G.;
    Connelly, A. Intrinsic non-stationarity correction for Fixel-Based Analysis.
    In Proc OHBM 2019 M789
    
    * If using the -nonstationary option:
    Salimi-Khorshidi, G. Smith, S.M. Nichols, T.E. Adjusting the effect of
    nonstationarity in cluster-based and TFCE inference. NeuroImage, 2011,
    54(3), 2006-19.
    
    Author: MRTrix3 Developers
    
    URL: https://www.mrtrix.org/
    
    Args:
        in_fixel_directory: the fixel directory containing the data files for\
            each subject (after obtaining fixel correspondence.
        subjects: a text file listing the subject identifiers (one per line).\
            This should correspond with the filenames in the fixel directory\
            (including the file extension), and be listed in the same order as the\
            rows of the design matrix.
        design: the design matrix.
        contrast: the contrast matrix, specified as rows of weights.
        connectivity: the fixel-fixel connectivity matrix.
        out_fixel_directory: the output directory where results will be saved.\
            Will be created if it does not exist.
        mask: provide a fixel data file containing a mask of those fixels to be\
            used during processing.
        notest: don't perform statistical inference; only output population\
            statistics (effect size, stdev etc).
        errors: specify nature of errors for shuffling; options are:\
            ee,ise,both (default: ee).
        exchange_within: specify blocks of observations within each of which\
            data may undergo restricted exchange.
        exchange_whole: specify blocks of observations that may be exchanged\
            with one another (for independent and symmetric errors, sign-flipping\
            will occur block-wise).
        strong: use strong familywise error control across multiple hypotheses.
        nshuffles: the number of shuffles (default: 5000).
        permutations: manually define the permutations (relabelling). The input\
            should be a text file defining a m x n matrix, where each relabelling\
            is defined as a column vector of size m, and the number of columns, n,\
            defines the number of permutations. Can be generated with the\
            palm_quickperms function in PALM\
            (http://fsl.fmrib.ox.ac.uk/fsl/fslwiki/PALM). Overrides the -nshuffles\
            option.
        nonstationarity: perform non-stationarity correction.
        skew_nonstationarity: specify the skew parameter for empirical\
            statistic calculation (default for this command is 1).
        nshuffles_nonstationarity: the number of shuffles to use when\
            precomputing the empirical statistic image for non-stationarity\
            correction (default: 5000).
        permutations_nonstationarity: manually define the permutations\
            (relabelling) for computing the emprical statistics for\
            non-stationarity correction. The input should be a text file defining a\
            m x n matrix, where each relabelling is defined as a column vector of\
            size m, and the number of columns, n, defines the number of\
            permutations. Can be generated with the palm_quickperms function in\
            PALM (http://fsl.fmrib.ox.ac.uk/fsl/fslwiki/PALM) Overrides the\
            -nshuffles_nonstationarity option.
        cfe_dh: the height increment used in the cfe integration (default: 0.1).
        cfe_e: cfe extent exponent (default: 2).
        cfe_h: cfe height exponent (default: 3).
        cfe_c: cfe connectivity exponent (default: 0.5).
        cfe_legacy: use the legacy (non-normalised) form of the cfe equation.
        variance: define variance groups for the G-statistic; measurements for\
            which the expected variance is equivalent should contain the same index.
        ftests: perform F-tests; input text file should contain, for each\
            F-test, a row containing ones and zeros, where ones indicate the rows\
            of the contrast matrix to be included in the F-test.
        fonly: only assess F-tests; do not perform statistical inference on\
            entries in the contrast matrix.
        column: add a column to the design matrix corresponding to subject\
            fixel-wise values (note that the contrast matrix must include an\
            additional column for each use of this option); the text file provided\
            via this option should contain a file name for each subject.
        info: display information messages.
        quiet: do not display information messages or progress status;\
            alternatively, this can be achieved by setting the MRTRIX_QUIET\
            environment variable to a non-empty string.
        debug: display debugging messages.
        force: force overwrite of output files (caution: using the same file as\
            input and output might cause unexpected behaviour).
        nthreads: use this number of threads in multi-threaded applications\
            (set to 0 to disable multi-threading).
        config: temporarily set the value of an MRtrix config file entry.
        help_: display this information page and exit.
        version: display version information and exit.
        runner: Command runner.
    Returns:
        NamedTuple of outputs (described in `FixelcfestatsOutputs`).
    """
    params = fixelcfestats_params(
        mask=mask,
        notest=notest,
        errors=errors,
        exchange_within=exchange_within,
        exchange_whole=exchange_whole,
        strong=strong,
        nshuffles=nshuffles,
        permutations=permutations,
        nonstationarity=nonstationarity,
        skew_nonstationarity=skew_nonstationarity,
        nshuffles_nonstationarity=nshuffles_nonstationarity,
        permutations_nonstationarity=permutations_nonstationarity,
        cfe_dh=cfe_dh,
        cfe_e=cfe_e,
        cfe_h=cfe_h,
        cfe_c=cfe_c,
        cfe_legacy=cfe_legacy,
        variance=variance,
        ftests=ftests,
        fonly=fonly,
        column=column,
        info=info,
        quiet=quiet,
        debug=debug,
        force=force,
        nthreads=nthreads,
        config=config,
        help_=help_,
        version=version,
        in_fixel_directory=in_fixel_directory,
        subjects=subjects,
        design=design,
        contrast=contrast,
        connectivity=connectivity,
        out_fixel_directory=out_fixel_directory,
    )
    return fixelcfestats_execute(params, runner)


__all__ = [
    "FIXELCFESTATS_METADATA",
    "FixelcfestatsColumnParamsDict",
    "FixelcfestatsColumnParamsDictTagged",
    "FixelcfestatsConfigParamsDict",
    "FixelcfestatsConfigParamsDictTagged",
    "FixelcfestatsOutputs",
    "FixelcfestatsParamsDict",
    "FixelcfestatsParamsDictTagged",
    "FixelcfestatsVariousFileParamsDict",
    "FixelcfestatsVariousFileParamsDictTagged",
    "FixelcfestatsVariousStringParamsDict",
    "FixelcfestatsVariousStringParamsDictTagged",
    "fixelcfestats",
    "fixelcfestats_column",
    "fixelcfestats_config",
    "fixelcfestats_execute",
    "fixelcfestats_params",
    "fixelcfestats_various_file",
    "fixelcfestats_various_string",
]



# This file was auto generated by Styx.
# Do not edit this file directly.

import typing
import pathlib
from styxdefs import *

MRFILTER_METADATA = Metadata(
    id="a47cabbb6dd1bf7eda885d594a87568625b0abbc.boutiques",
    name="mrfilter",
    package="mrtrix",
    container_image_tag="mrtrix3/mrtrix3:3.0.4",
)


_MrfilterVariousStringParamsDictNoTag = typing.TypedDict('_MrfilterVariousStringParamsDictNoTag', {
    "obj": str,
})
MrfilterVariousStringParamsDictTagged = typing.TypedDict('MrfilterVariousStringParamsDictTagged', {
    "@type": typing.Literal["VariousString"],
    "obj": str,
})
MrfilterVariousStringParamsDict = _MrfilterVariousStringParamsDictNoTag | MrfilterVariousStringParamsDictTagged


_MrfilterVariousFileParamsDictNoTag = typing.TypedDict('_MrfilterVariousFileParamsDictNoTag', {
    "obj": InputPathType,
})
MrfilterVariousFileParamsDictTagged = typing.TypedDict('MrfilterVariousFileParamsDictTagged', {
    "@type": typing.Literal["VariousFile"],
    "obj": InputPathType,
})
MrfilterVariousFileParamsDict = _MrfilterVariousFileParamsDictNoTag | MrfilterVariousFileParamsDictTagged


_MrfilterConfigParamsDictNoTag = typing.TypedDict('_MrfilterConfigParamsDictNoTag', {
    "key": str,
    "value": str,
})
MrfilterConfigParamsDictTagged = typing.TypedDict('MrfilterConfigParamsDictTagged', {
    "@type": typing.Literal["config"],
    "key": str,
    "value": str,
})
MrfilterConfigParamsDict = _MrfilterConfigParamsDictNoTag | MrfilterConfigParamsDictTagged


_MrfilterParamsDictNoTag = typing.TypedDict('_MrfilterParamsDictNoTag', {
    "axes": typing.NotRequired[list[int] | None],
    "inverse": bool,
    "magnitude": bool,
    "centre_zero": bool,
    "stdev": typing.NotRequired[list[float] | None],
    "magnitude_1": bool,
    "scanner": bool,
    "extent": typing.NotRequired[list[int] | None],
    "extent_1": typing.NotRequired[list[int] | None],
    "stdev_1": typing.NotRequired[list[float] | None],
    "fwhm": typing.NotRequired[list[float] | None],
    "extent_2": typing.NotRequired[list[int] | None],
    "zupper": typing.NotRequired[float | None],
    "zlower": typing.NotRequired[float | None],
    "bridge": typing.NotRequired[int | None],
    "maskin": typing.NotRequired[InputPathType | None],
    "maskout": typing.NotRequired[str | None],
    "strides": typing.NotRequired[typing.Union[MrfilterVariousStringParamsDictTagged, MrfilterVariousFileParamsDictTagged] | None],
    "info": bool,
    "quiet": bool,
    "debug": bool,
    "force": bool,
    "nthreads": typing.NotRequired[int | None],
    "config": typing.NotRequired[list[MrfilterConfigParamsDict] | None],
    "help": bool,
    "version": bool,
    "input": InputPathType,
    "filter": str,
    "output": str,
})
MrfilterParamsDictTagged = typing.TypedDict('MrfilterParamsDictTagged', {
    "@type": typing.Literal["mrtrix/mrfilter"],
    "axes": typing.NotRequired[list[int] | None],
    "inverse": bool,
    "magnitude": bool,
    "centre_zero": bool,
    "stdev": typing.NotRequired[list[float] | None],
    "magnitude_1": bool,
    "scanner": bool,
    "extent": typing.NotRequired[list[int] | None],
    "extent_1": typing.NotRequired[list[int] | None],
    "stdev_1": typing.NotRequired[list[float] | None],
    "fwhm": typing.NotRequired[list[float] | None],
    "extent_2": typing.NotRequired[list[int] | None],
    "zupper": typing.NotRequired[float | None],
    "zlower": typing.NotRequired[float | None],
    "bridge": typing.NotRequired[int | None],
    "maskin": typing.NotRequired[InputPathType | None],
    "maskout": typing.NotRequired[str | None],
    "strides": typing.NotRequired[typing.Union[MrfilterVariousStringParamsDictTagged, MrfilterVariousFileParamsDictTagged] | None],
    "info": bool,
    "quiet": bool,
    "debug": bool,
    "force": bool,
    "nthreads": typing.NotRequired[int | None],
    "config": typing.NotRequired[list[MrfilterConfigParamsDict] | None],
    "help": bool,
    "version": bool,
    "input": InputPathType,
    "filter": str,
    "output": str,
})
MrfilterParamsDict = _MrfilterParamsDictNoTag | MrfilterParamsDictTagged


def mrfilter_strides_cargs_dyn_fn(
    t: str,
) -> typing.Any:
    """
    Get build cargs function by command type.
    
    Args:
        t: Command type.
    Returns:
        Build cargs function.
    """
    return {
        "VariousString": mrfilter_various_string_cargs,
        "VariousFile": mrfilter_various_file_cargs,
    }.get(t)


def mrfilter_strides_validate_dyn_fn(
    t: str,
) -> typing.Any:
    """
    Get validate params function by command type.
    
    Args:
        t: Command type.
    Returns:
        Validate params function.
    """
    return {
        "VariousString": mrfilter_various_string_validate,
        "VariousFile": mrfilter_various_file_validate,
    }.get(t)


def mrfilter_various_string(
    obj: str,
) -> MrfilterVariousStringParamsDictTagged:
    """
    Build parameters.
    
    Args:
        obj: String object.
    Returns:
        Parameter dictionary
    """
    params = {
        "@type": "VariousString",
        "obj": obj,
    }
    return params


def mrfilter_various_string_validate(
    params: typing.Any,
) -> None:
    """
    Validate parameters. Throws an error if `params` is not a valid
    `MrfilterVariousStringParamsDict` object.
    
    Args:
        params: The parameters object to validate.
    """
    if params is None or not isinstance(params, dict):
        raise StyxValidationError(f'Params object has the wrong type \'{type(params)}\'')
    if params.get("obj", None) is None:
        raise StyxValidationError("`obj` must not be None")
    if not isinstance(params["obj"], str):
        raise StyxValidationError(f'`obj` has the wrong type: Received `{type(params.get("obj", None))}` expected `str`')


def mrfilter_various_string_cargs(
    params: MrfilterVariousStringParamsDict,
    execution: Execution,
) -> list[str]:
    """
    Build command-line arguments from parameters.
    
    Args:
        params: The parameters.
        execution: The execution object for resolving input paths.
    Returns:
        Command-line arguments.
    """
    cargs = []
    cargs.append(params.get("obj", None))
    return cargs


def mrfilter_various_file(
    obj: InputPathType,
) -> MrfilterVariousFileParamsDictTagged:
    """
    Build parameters.
    
    Args:
        obj: File object.
    Returns:
        Parameter dictionary
    """
    params = {
        "@type": "VariousFile",
        "obj": obj,
    }
    return params


def mrfilter_various_file_validate(
    params: typing.Any,
) -> None:
    """
    Validate parameters. Throws an error if `params` is not a valid
    `MrfilterVariousFileParamsDict` object.
    
    Args:
        params: The parameters object to validate.
    """
    if params is None or not isinstance(params, dict):
        raise StyxValidationError(f'Params object has the wrong type \'{type(params)}\'')
    if params.get("obj", None) is None:
        raise StyxValidationError("`obj` must not be None")
    if not isinstance(params["obj"], (pathlib.Path, str)):
        raise StyxValidationError(f'`obj` has the wrong type: Received `{type(params.get("obj", None))}` expected `InputPathType`')


def mrfilter_various_file_cargs(
    params: MrfilterVariousFileParamsDict,
    execution: Execution,
) -> list[str]:
    """
    Build command-line arguments from parameters.
    
    Args:
        params: The parameters.
        execution: The execution object for resolving input paths.
    Returns:
        Command-line arguments.
    """
    cargs = []
    cargs.append(execution.input_file(params.get("obj", None)))
    return cargs


def mrfilter_config(
    key: str,
    value: str,
) -> MrfilterConfigParamsDictTagged:
    """
    Build parameters.
    
    Args:
        key: temporarily set the value of an MRtrix config file entry.
        value: temporarily set the value of an MRtrix config file entry.
    Returns:
        Parameter dictionary
    """
    params = {
        "@type": "config",
        "key": key,
        "value": value,
    }
    return params


def mrfilter_config_validate(
    params: typing.Any,
) -> None:
    """
    Validate parameters. Throws an error if `params` is not a valid
    `MrfilterConfigParamsDict` object.
    
    Args:
        params: The parameters object to validate.
    """
    if params is None or not isinstance(params, dict):
        raise StyxValidationError(f'Params object has the wrong type \'{type(params)}\'')
    if params.get("key", None) is None:
        raise StyxValidationError("`key` must not be None")
    if not isinstance(params["key"], str):
        raise StyxValidationError(f'`key` has the wrong type: Received `{type(params.get("key", None))}` expected `str`')
    if params.get("value", None) is None:
        raise StyxValidationError("`value` must not be None")
    if not isinstance(params["value"], str):
        raise StyxValidationError(f'`value` has the wrong type: Received `{type(params.get("value", None))}` expected `str`')


def mrfilter_config_cargs(
    params: MrfilterConfigParamsDict,
    execution: Execution,
) -> list[str]:
    """
    Build command-line arguments from parameters.
    
    Args:
        params: The parameters.
        execution: The execution object for resolving input paths.
    Returns:
        Command-line arguments.
    """
    cargs = []
    cargs.append("-config")
    cargs.append(params.get("key", None))
    cargs.append(params.get("value", None))
    return cargs


class MrfilterOutputs(typing.NamedTuple):
    """
    Output object returned when calling `MrfilterParamsDict(...)`.
    """
    root: OutputPathType
    """Output root folder. This is the root folder for all outputs."""
    output: OutputPathType
    """the output image."""
    maskout: OutputPathType | None
    """Output a refined mask based on a spatially coherent region with normal
    intensity range. """


def mrfilter_params(
    input_: InputPathType,
    filter_: str,
    output: str,
    axes: list[int] | None = None,
    inverse: bool = False,
    magnitude: bool = False,
    centre_zero: bool = False,
    stdev: list[float] | None = None,
    magnitude_1: bool = False,
    scanner: bool = False,
    extent: list[int] | None = None,
    extent_1: list[int] | None = None,
    stdev_1: list[float] | None = None,
    fwhm: list[float] | None = None,
    extent_2: list[int] | None = None,
    zupper: float | None = None,
    zlower: float | None = None,
    bridge: int | None = None,
    maskin: InputPathType | None = None,
    maskout: str | None = None,
    strides: typing.Union[MrfilterVariousStringParamsDictTagged, MrfilterVariousFileParamsDictTagged] | None = None,
    info: bool = False,
    quiet: bool = False,
    debug: bool = False,
    force: bool = False,
    nthreads: int | None = None,
    config: list[MrfilterConfigParamsDict] | None = None,
    help_: bool = False,
    version: bool = False,
) -> MrfilterParamsDictTagged:
    """
    Build parameters.
    
    Args:
        input_: the input image.
        filter_: the type of filter to be applied.
        output: the output image.
        axes: the axes along which to apply the Fourier Transform. By default,\
            the transform is applied along the three spatial axes. Provide as a\
            comma-separate list of axis indices.
        inverse: apply the inverse FFT.
        magnitude: output a magnitude image rather than a complex-valued image.
        centre_zero: re-arrange the FFT results so that the zero-frequency\
            component appears in the centre of the image, rather than at the edges.
        stdev: the standard deviation of the Gaussian kernel used to smooth the\
            input image (in mm). The image is smoothed to reduced large spurious\
            gradients caused by noise. Use this option to override the default\
            stdev of 1 voxel. This can be specified either as a single value to be\
            used for all 3 axes, or as a comma-separated list of 3 values, one for\
            each axis.
        magnitude_1: output the gradient magnitude, rather than the default\
            x,y,z components.
        scanner: define the gradient with respect to the scanner coordinate\
            frame of reference.
        extent: specify extent of median filtering neighbourhood in voxels.\
            This can be specified either as a single value to be used for all 3\
            axes, or as a comma-separated list of 3 values, one for each axis\
            (default: 3x3x3).
        extent_1: specify extent of normalisation filtering neighbourhood in\
            voxels. This can be specified either as a single value to be used for\
            all 3 axes, or as a comma-separated list of 3 values, one for each axis\
            (default: 3x3x3).
        stdev_1: apply Gaussian smoothing with the specified standard\
            deviation. The standard deviation is defined in mm (Default 1 voxel).\
            This can be specified either as a single value to be used for all axes,\
            or as a comma-separated list of the stdev for each axis.
        fwhm: apply Gaussian smoothing with the specified full-width half\
            maximum. The FWHM is defined in mm (Default 1 voxel * 2.3548). This can\
            be specified either as a single value to be used for all axes, or as a\
            comma-separated list of the FWHM for each axis.
        extent_2: specify the extent (width) of kernel size in voxels. This can\
            be specified either as a single value to be used for all axes, or as a\
            comma-separated list of the extent for each axis. The default extent is\
            2 * ceil(2.5 * stdev / voxel_size) - 1.
        zupper: define high intensity outliers: default: 2.5.
        zlower: define low intensity outliers: default: 2.5.
        bridge: number of voxels to gap to fill holes in mask: default: 4.
        maskin: initial mask that defines the maximum spatial extent and the\
            region from which to smaple the intensity range.
        maskout: Output a refined mask based on a spatially coherent region\
            with normal intensity range.
        strides: specify the strides of the output data in memory; either as a\
            comma-separated list of (signed) integers, or as a template image from\
            which the strides shall be extracted and used. The actual strides\
            produced will depend on whether the output image format can support it.
        info: display information messages.
        quiet: do not display information messages or progress status;\
            alternatively, this can be achieved by setting the MRTRIX_QUIET\
            environment variable to a non-empty string.
        debug: display debugging messages.
        force: force overwrite of output files (caution: using the same file as\
            input and output might cause unexpected behaviour).
        nthreads: use this number of threads in multi-threaded applications\
            (set to 0 to disable multi-threading).
        config: temporarily set the value of an MRtrix config file entry.
        help_: display this information page and exit.
        version: display version information and exit.
    Returns:
        Parameter dictionary
    """
    params = {
        "@type": "mrtrix/mrfilter",
        "inverse": inverse,
        "magnitude": magnitude,
        "centre_zero": centre_zero,
        "magnitude_1": magnitude_1,
        "scanner": scanner,
        "info": info,
        "quiet": quiet,
        "debug": debug,
        "force": force,
        "help": help_,
        "version": version,
        "input": input_,
        "filter": filter_,
        "output": output,
    }
    if axes is not None:
        params["axes"] = axes
    if stdev is not None:
        params["stdev"] = stdev
    if extent is not None:
        params["extent"] = extent
    if extent_1 is not None:
        params["extent_1"] = extent_1
    if stdev_1 is not None:
        params["stdev_1"] = stdev_1
    if fwhm is not None:
        params["fwhm"] = fwhm
    if extent_2 is not None:
        params["extent_2"] = extent_2
    if zupper is not None:
        params["zupper"] = zupper
    if zlower is not None:
        params["zlower"] = zlower
    if bridge is not None:
        params["bridge"] = bridge
    if maskin is not None:
        params["maskin"] = maskin
    if maskout is not None:
        params["maskout"] = maskout
    if strides is not None:
        params["strides"] = strides
    if nthreads is not None:
        params["nthreads"] = nthreads
    if config is not None:
        params["config"] = config
    return params


def mrfilter_validate(
    params: typing.Any,
) -> None:
    """
    Validate parameters. Throws an error if `params` is not a valid
    `MrfilterParamsDict` object.
    
    Args:
        params: The parameters object to validate.
    """
    if params is None or not isinstance(params, dict):
        raise StyxValidationError(f'Params object has the wrong type \'{type(params)}\'')
    if params.get("axes", None) is not None:
        if not isinstance(params["axes"], list):
            raise StyxValidationError(f'`axes` has the wrong type: Received `{type(params.get("axes", None))}` expected `list[int] | None`')
        for e in params["axes"]:
            if not isinstance(e, int):
                raise StyxValidationError(f'`axes` has the wrong type: Received `{type(params.get("axes", None))}` expected `list[int] | None`')
    if params.get("inverse", False) is None:
        raise StyxValidationError("`inverse` must not be None")
    if not isinstance(params["inverse"], bool):
        raise StyxValidationError(f'`inverse` has the wrong type: Received `{type(params.get("inverse", False))}` expected `bool`')
    if params.get("magnitude", False) is None:
        raise StyxValidationError("`magnitude` must not be None")
    if not isinstance(params["magnitude"], bool):
        raise StyxValidationError(f'`magnitude` has the wrong type: Received `{type(params.get("magnitude", False))}` expected `bool`')
    if params.get("centre_zero", False) is None:
        raise StyxValidationError("`centre_zero` must not be None")
    if not isinstance(params["centre_zero"], bool):
        raise StyxValidationError(f'`centre_zero` has the wrong type: Received `{type(params.get("centre_zero", False))}` expected `bool`')
    if params.get("stdev", None) is not None:
        if not isinstance(params["stdev"], list):
            raise StyxValidationError(f'`stdev` has the wrong type: Received `{type(params.get("stdev", None))}` expected `list[float] | None`')
        for e in params["stdev"]:
            if not isinstance(e, (float, int)):
                raise StyxValidationError(f'`stdev` has the wrong type: Received `{type(params.get("stdev", None))}` expected `list[float] | None`')
    if params.get("magnitude_1", False) is None:
        raise StyxValidationError("`magnitude_1` must not be None")
    if not isinstance(params["magnitude_1"], bool):
        raise StyxValidationError(f'`magnitude_1` has the wrong type: Received `{type(params.get("magnitude_1", False))}` expected `bool`')
    if params.get("scanner", False) is None:
        raise StyxValidationError("`scanner` must not be None")
    if not isinstance(params["scanner"], bool):
        raise StyxValidationError(f'`scanner` has the wrong type: Received `{type(params.get("scanner", False))}` expected `bool`')
    if params.get("extent", None) is not None:
        if not isinstance(params["extent"], list):
            raise StyxValidationError(f'`extent` has the wrong type: Received `{type(params.get("extent", None))}` expected `list[int] | None`')
        for e in params["extent"]:
            if not isinstance(e, int):
                raise StyxValidationError(f'`extent` has the wrong type: Received `{type(params.get("extent", None))}` expected `list[int] | None`')
    if params.get("extent_1", None) is not None:
        if not isinstance(params["extent_1"], list):
            raise StyxValidationError(f'`extent_1` has the wrong type: Received `{type(params.get("extent_1", None))}` expected `list[int] | None`')
        for e in params["extent_1"]:
            if not isinstance(e, int):
                raise StyxValidationError(f'`extent_1` has the wrong type: Received `{type(params.get("extent_1", None))}` expected `list[int] | None`')
    if params.get("stdev_1", None) is not None:
        if not isinstance(params["stdev_1"], list):
            raise StyxValidationError(f'`stdev_1` has the wrong type: Received `{type(params.get("stdev_1", None))}` expected `list[float] | None`')
        for e in params["stdev_1"]:
            if not isinstance(e, (float, int)):
                raise StyxValidationError(f'`stdev_1` has the wrong type: Received `{type(params.get("stdev_1", None))}` expected `list[float] | None`')
    if params.get("fwhm", None) is not None:
        if not isinstance(params["fwhm"], list):
            raise StyxValidationError(f'`fwhm` has the wrong type: Received `{type(params.get("fwhm", None))}` expected `list[float] | None`')
        for e in params["fwhm"]:
            if not isinstance(e, (float, int)):
                raise StyxValidationError(f'`fwhm` has the wrong type: Received `{type(params.get("fwhm", None))}` expected `list[float] | None`')
    if params.get("extent_2", None) is not None:
        if not isinstance(params["extent_2"], list):
            raise StyxValidationError(f'`extent_2` has the wrong type: Received `{type(params.get("extent_2", None))}` expected `list[int] | None`')
        for e in params["extent_2"]:
            if not isinstance(e, int):
                raise StyxValidationError(f'`extent_2` has the wrong type: Received `{type(params.get("extent_2", None))}` expected `list[int] | None`')
    if params.get("zupper", None) is not None:
        if not isinstance(params["zupper"], (float, int)):
            raise StyxValidationError(f'`zupper` has the wrong type: Received `{type(params.get("zupper", None))}` expected `float | None`')
    if params.get("zlower", None) is not None:
        if not isinstance(params["zlower"], (float, int)):
            raise StyxValidationError(f'`zlower` has the wrong type: Received `{type(params.get("zlower", None))}` expected `float | None`')
    if params.get("bridge", None) is not None:
        if not isinstance(params["bridge"], int):
            raise StyxValidationError(f'`bridge` has the wrong type: Received `{type(params.get("bridge", None))}` expected `int | None`')
    if params.get("maskin", None) is not None:
        if not isinstance(params["maskin"], (pathlib.Path, str)):
            raise StyxValidationError(f'`maskin` has the wrong type: Received `{type(params.get("maskin", None))}` expected `InputPathType | None`')
    if params.get("maskout", None) is not None:
        if not isinstance(params["maskout"], str):
            raise StyxValidationError(f'`maskout` has the wrong type: Received `{type(params.get("maskout", None))}` expected `str | None`')
    if params.get("strides", None) is not None:
        if not isinstance(params["strides"], dict):
            raise StyxValidationError(f'Params object has the wrong type \'{type(params["strides"])}\'')
        if "@type" not in params["strides"]:
            raise StyxValidationError("Params object is missing `@type`")
        if params["strides"]["@type"] not in ["VariousString", "VariousFile"]:
            raise StyxValidationError("Parameter `strides`s `@type` must be one of [\"VariousString\", \"VariousFile\"]")
        mrfilter_strides_validate_dyn_fn(params["strides"]["@type"])(params["strides"])
    if params.get("info", False) is None:
        raise StyxValidationError("`info` must not be None")
    if not isinstance(params["info"], bool):
        raise StyxValidationError(f'`info` has the wrong type: Received `{type(params.get("info", False))}` expected `bool`')
    if params.get("quiet", False) is None:
        raise StyxValidationError("`quiet` must not be None")
    if not isinstance(params["quiet"], bool):
        raise StyxValidationError(f'`quiet` has the wrong type: Received `{type(params.get("quiet", False))}` expected `bool`')
    if params.get("debug", False) is None:
        raise StyxValidationError("`debug` must not be None")
    if not isinstance(params["debug"], bool):
        raise StyxValidationError(f'`debug` has the wrong type: Received `{type(params.get("debug", False))}` expected `bool`')
    if params.get("force", False) is None:
        raise StyxValidationError("`force` must not be None")
    if not isinstance(params["force"], bool):
        raise StyxValidationError(f'`force` has the wrong type: Received `{type(params.get("force", False))}` expected `bool`')
    if params.get("nthreads", None) is not None:
        if not isinstance(params["nthreads"], int):
            raise StyxValidationError(f'`nthreads` has the wrong type: Received `{type(params.get("nthreads", None))}` expected `int | None`')
    if params.get("config", None) is not None:
        if not isinstance(params["config"], list):
            raise StyxValidationError(f'`config` has the wrong type: Received `{type(params.get("config", None))}` expected `list[MrfilterConfigParamsDict] | None`')
        for e in params["config"]:
            mrfilter_config_validate(e)
    if params.get("help", False) is None:
        raise StyxValidationError("`help` must not be None")
    if not isinstance(params["help"], bool):
        raise StyxValidationError(f'`help` has the wrong type: Received `{type(params.get("help", False))}` expected `bool`')
    if params.get("version", False) is None:
        raise StyxValidationError("`version` must not be None")
    if not isinstance(params["version"], bool):
        raise StyxValidationError(f'`version` has the wrong type: Received `{type(params.get("version", False))}` expected `bool`')
    if params.get("input", None) is None:
        raise StyxValidationError("`input` must not be None")
    if not isinstance(params["input"], (pathlib.Path, str)):
        raise StyxValidationError(f'`input` has the wrong type: Received `{type(params.get("input", None))}` expected `InputPathType`')
    if params.get("filter", None) is None:
        raise StyxValidationError("`filter` must not be None")
    if not isinstance(params["filter"], str):
        raise StyxValidationError(f'`filter` has the wrong type: Received `{type(params.get("filter", None))}` expected `str`')
    if params.get("output", None) is None:
        raise StyxValidationError("`output` must not be None")
    if not isinstance(params["output"], str):
        raise StyxValidationError(f'`output` has the wrong type: Received `{type(params.get("output", None))}` expected `str`')


def mrfilter_cargs(
    params: MrfilterParamsDict,
    execution: Execution,
) -> list[str]:
    """
    Build command-line arguments from parameters.
    
    Args:
        params: The parameters.
        execution: The execution object for resolving input paths.
    Returns:
        Command-line arguments.
    """
    cargs = []
    cargs.append("mrfilter")
    if params.get("axes", None) is not None:
        cargs.extend([
            "-axes",
            *map(str, params.get("axes", None))
        ])
    if params.get("inverse", False):
        cargs.append("-inverse")
    if params.get("magnitude", False):
        cargs.append("-magnitude")
    if params.get("centre_zero", False):
        cargs.append("-centre_zero")
    if params.get("stdev", None) is not None:
        cargs.extend([
            "-stdev",
            ",".join(map(str, params.get("stdev", None)))
        ])
    if params.get("magnitude_1", False):
        cargs.append("-magnitude")
    if params.get("scanner", False):
        cargs.append("-scanner")
    if params.get("extent", None) is not None:
        cargs.extend([
            "-extent",
            ",".join(map(str, params.get("extent", None)))
        ])
    if params.get("extent_1", None) is not None:
        cargs.extend([
            "-extent",
            ",".join(map(str, params.get("extent_1", None)))
        ])
    if params.get("stdev_1", None) is not None:
        cargs.extend([
            "-stdev",
            ",".join(map(str, params.get("stdev_1", None)))
        ])
    if params.get("fwhm", None) is not None:
        cargs.extend([
            "-fwhm",
            ",".join(map(str, params.get("fwhm", None)))
        ])
    if params.get("extent_2", None) is not None:
        cargs.extend([
            "-extent",
            ",".join(map(str, params.get("extent_2", None)))
        ])
    if params.get("zupper", None) is not None:
        cargs.extend([
            "-zupper",
            str(params.get("zupper", None))
        ])
    if params.get("zlower", None) is not None:
        cargs.extend([
            "-zlower",
            str(params.get("zlower", None))
        ])
    if params.get("bridge", None) is not None:
        cargs.extend([
            "-bridge",
            str(params.get("bridge", None))
        ])
    if params.get("maskin", None) is not None:
        cargs.extend([
            "-maskin",
            execution.input_file(params.get("maskin", None))
        ])
    if params.get("maskout", None) is not None:
        cargs.extend([
            "-maskout",
            params.get("maskout", None)
        ])
    if params.get("strides", None) is not None:
        cargs.extend([
            "-strides",
            *mrfilter_strides_cargs_dyn_fn(params.get("strides", None)["@type"])(params.get("strides", None), execution)
        ])
    if params.get("info", False):
        cargs.append("-info")
    if params.get("quiet", False):
        cargs.append("-quiet")
    if params.get("debug", False):
        cargs.append("-debug")
    if params.get("force", False):
        cargs.append("-force")
    if params.get("nthreads", None) is not None:
        cargs.extend([
            "-nthreads",
            str(params.get("nthreads", None))
        ])
    if params.get("config", None) is not None:
        cargs.extend([a for c in [mrfilter_config_cargs(s, execution) for s in params.get("config", None)] for a in c])
    if params.get("help", False):
        cargs.append("-help")
    if params.get("version", False):
        cargs.append("-version")
    cargs.append(execution.input_file(params.get("input", None)))
    cargs.append(params.get("filter", None))
    cargs.append(params.get("output", None))
    return cargs


def mrfilter_outputs(
    params: MrfilterParamsDict,
    execution: Execution,
) -> MrfilterOutputs:
    """
    Build outputs object containing output file paths and possibly stdout/stderr.
    
    Args:
        params: The parameters.
        execution: The execution object for resolving input paths.
    Returns:
        Outputs object.
    """
    ret = MrfilterOutputs(
        root=execution.output_file("."),
        output=execution.output_file(params.get("output", None)),
        maskout=execution.output_file(params.get("maskout", None)) if (params.get("maskout") is not None) else None,
    )
    return ret


def mrfilter_execute(
    params: MrfilterParamsDict,
    runner: Runner | None = None,
) -> MrfilterOutputs:
    """
    mrfilter
    
    Perform filtering operations on 3D / 4D MR images.
    
    The available filters are: fft, gradient, median, smooth, normalise, zclean.
    
    Each filter has its own unique set of optional parameters.
    
    For 4D images, each 3D volume is processed independently.
    
    References:
    
    .
    
    Author: MRTrix3 Developers
    
    URL: https://www.mrtrix.org/
    
    Args:
        params: The parameters.
        runner: Command runner.
    Returns:
        NamedTuple of outputs (described in `MrfilterOutputs`).
    """
    mrfilter_validate(params)
    runner = runner or get_global_runner()
    execution = runner.start_execution(MRFILTER_METADATA)
    params = execution.params(params)
    cargs = mrfilter_cargs(params, execution)
    ret = mrfilter_outputs(params, execution)
    execution.run(cargs)
    return ret


def mrfilter(
    input_: InputPathType,
    filter_: str,
    output: str,
    axes: list[int] | None = None,
    inverse: bool = False,
    magnitude: bool = False,
    centre_zero: bool = False,
    stdev: list[float] | None = None,
    magnitude_1: bool = False,
    scanner: bool = False,
    extent: list[int] | None = None,
    extent_1: list[int] | None = None,
    stdev_1: list[float] | None = None,
    fwhm: list[float] | None = None,
    extent_2: list[int] | None = None,
    zupper: float | None = None,
    zlower: float | None = None,
    bridge: int | None = None,
    maskin: InputPathType | None = None,
    maskout: str | None = None,
    strides: typing.Union[MrfilterVariousStringParamsDictTagged, MrfilterVariousFileParamsDictTagged] | None = None,
    info: bool = False,
    quiet: bool = False,
    debug: bool = False,
    force: bool = False,
    nthreads: int | None = None,
    config: list[MrfilterConfigParamsDict] | None = None,
    help_: bool = False,
    version: bool = False,
    runner: Runner | None = None,
) -> MrfilterOutputs:
    """
    mrfilter
    
    Perform filtering operations on 3D / 4D MR images.
    
    The available filters are: fft, gradient, median, smooth, normalise, zclean.
    
    Each filter has its own unique set of optional parameters.
    
    For 4D images, each 3D volume is processed independently.
    
    References:
    
    .
    
    Author: MRTrix3 Developers
    
    URL: https://www.mrtrix.org/
    
    Args:
        input_: the input image.
        filter_: the type of filter to be applied.
        output: the output image.
        axes: the axes along which to apply the Fourier Transform. By default,\
            the transform is applied along the three spatial axes. Provide as a\
            comma-separate list of axis indices.
        inverse: apply the inverse FFT.
        magnitude: output a magnitude image rather than a complex-valued image.
        centre_zero: re-arrange the FFT results so that the zero-frequency\
            component appears in the centre of the image, rather than at the edges.
        stdev: the standard deviation of the Gaussian kernel used to smooth the\
            input image (in mm). The image is smoothed to reduced large spurious\
            gradients caused by noise. Use this option to override the default\
            stdev of 1 voxel. This can be specified either as a single value to be\
            used for all 3 axes, or as a comma-separated list of 3 values, one for\
            each axis.
        magnitude_1: output the gradient magnitude, rather than the default\
            x,y,z components.
        scanner: define the gradient with respect to the scanner coordinate\
            frame of reference.
        extent: specify extent of median filtering neighbourhood in voxels.\
            This can be specified either as a single value to be used for all 3\
            axes, or as a comma-separated list of 3 values, one for each axis\
            (default: 3x3x3).
        extent_1: specify extent of normalisation filtering neighbourhood in\
            voxels. This can be specified either as a single value to be used for\
            all 3 axes, or as a comma-separated list of 3 values, one for each axis\
            (default: 3x3x3).
        stdev_1: apply Gaussian smoothing with the specified standard\
            deviation. The standard deviation is defined in mm (Default 1 voxel).\
            This can be specified either as a single value to be used for all axes,\
            or as a comma-separated list of the stdev for each axis.
        fwhm: apply Gaussian smoothing with the specified full-width half\
            maximum. The FWHM is defined in mm (Default 1 voxel * 2.3548). This can\
            be specified either as a single value to be used for all axes, or as a\
            comma-separated list of the FWHM for each axis.
        extent_2: specify the extent (width) of kernel size in voxels. This can\
            be specified either as a single value to be used for all axes, or as a\
            comma-separated list of the extent for each axis. The default extent is\
            2 * ceil(2.5 * stdev / voxel_size) - 1.
        zupper: define high intensity outliers: default: 2.5.
        zlower: define low intensity outliers: default: 2.5.
        bridge: number of voxels to gap to fill holes in mask: default: 4.
        maskin: initial mask that defines the maximum spatial extent and the\
            region from which to smaple the intensity range.
        maskout: Output a refined mask based on a spatially coherent region\
            with normal intensity range.
        strides: specify the strides of the output data in memory; either as a\
            comma-separated list of (signed) integers, or as a template image from\
            which the strides shall be extracted and used. The actual strides\
            produced will depend on whether the output image format can support it.
        info: display information messages.
        quiet: do not display information messages or progress status;\
            alternatively, this can be achieved by setting the MRTRIX_QUIET\
            environment variable to a non-empty string.
        debug: display debugging messages.
        force: force overwrite of output files (caution: using the same file as\
            input and output might cause unexpected behaviour).
        nthreads: use this number of threads in multi-threaded applications\
            (set to 0 to disable multi-threading).
        config: temporarily set the value of an MRtrix config file entry.
        help_: display this information page and exit.
        version: display version information and exit.
        runner: Command runner.
    Returns:
        NamedTuple of outputs (described in `MrfilterOutputs`).
    """
    params = mrfilter_params(
        axes=axes,
        inverse=inverse,
        magnitude=magnitude,
        centre_zero=centre_zero,
        stdev=stdev,
        magnitude_1=magnitude_1,
        scanner=scanner,
        extent=extent,
        extent_1=extent_1,
        stdev_1=stdev_1,
        fwhm=fwhm,
        extent_2=extent_2,
        zupper=zupper,
        zlower=zlower,
        bridge=bridge,
        maskin=maskin,
        maskout=maskout,
        strides=strides,
        info=info,
        quiet=quiet,
        debug=debug,
        force=force,
        nthreads=nthreads,
        config=config,
        help_=help_,
        version=version,
        input_=input_,
        filter_=filter_,
        output=output,
    )
    return mrfilter_execute(params, runner)


__all__ = [
    "MRFILTER_METADATA",
    "MrfilterConfigParamsDict",
    "MrfilterConfigParamsDictTagged",
    "MrfilterOutputs",
    "MrfilterParamsDict",
    "MrfilterParamsDictTagged",
    "MrfilterVariousFileParamsDict",
    "MrfilterVariousFileParamsDictTagged",
    "MrfilterVariousStringParamsDict",
    "MrfilterVariousStringParamsDictTagged",
    "mrfilter",
    "mrfilter_config",
    "mrfilter_execute",
    "mrfilter_params",
    "mrfilter_various_file",
    "mrfilter_various_string",
]



# This file was auto generated by Styx.
# Do not edit this file directly.

import typing
import pathlib
from styxdefs import *

MRCOLOUR_METADATA = Metadata(
    id="9cd4b4f06262529bb32fb59ee13631275aef99fa.boutiques",
    name="mrcolour",
    package="mrtrix",
    container_image_tag="mrtrix3/mrtrix3:3.0.4",
)


_MrcolourConfigParamsDictNoTag = typing.TypedDict('_MrcolourConfigParamsDictNoTag', {
    "key": str,
    "value": str,
})
MrcolourConfigParamsDictTagged = typing.TypedDict('MrcolourConfigParamsDictTagged', {
    "@type": typing.Literal["config"],
    "key": str,
    "value": str,
})
MrcolourConfigParamsDict = _MrcolourConfigParamsDictNoTag | MrcolourConfigParamsDictTagged


_MrcolourParamsDictNoTag = typing.TypedDict('_MrcolourParamsDictNoTag', {
    "upper": typing.NotRequired[float | None],
    "lower": typing.NotRequired[float | None],
    "colour": typing.NotRequired[list[float] | None],
    "info": bool,
    "quiet": bool,
    "debug": bool,
    "force": bool,
    "nthreads": typing.NotRequired[int | None],
    "config": typing.NotRequired[list[MrcolourConfigParamsDict] | None],
    "help": bool,
    "version": bool,
    "input": InputPathType,
    "map": str,
    "output": str,
})
MrcolourParamsDictTagged = typing.TypedDict('MrcolourParamsDictTagged', {
    "@type": typing.Literal["mrtrix/mrcolour"],
    "upper": typing.NotRequired[float | None],
    "lower": typing.NotRequired[float | None],
    "colour": typing.NotRequired[list[float] | None],
    "info": bool,
    "quiet": bool,
    "debug": bool,
    "force": bool,
    "nthreads": typing.NotRequired[int | None],
    "config": typing.NotRequired[list[MrcolourConfigParamsDict] | None],
    "help": bool,
    "version": bool,
    "input": InputPathType,
    "map": str,
    "output": str,
})
MrcolourParamsDict = _MrcolourParamsDictNoTag | MrcolourParamsDictTagged


def mrcolour_config(
    key: str,
    value: str,
) -> MrcolourConfigParamsDictTagged:
    """
    Build parameters.
    
    Args:
        key: temporarily set the value of an MRtrix config file entry.
        value: temporarily set the value of an MRtrix config file entry.
    Returns:
        Parameter dictionary
    """
    params = {
        "@type": "config",
        "key": key,
        "value": value,
    }
    return params


def mrcolour_config_validate(
    params: typing.Any,
) -> None:
    """
    Validate parameters. Throws an error if `params` is not a valid
    `MrcolourConfigParamsDict` object.
    
    Args:
        params: The parameters object to validate.
    """
    if params is None or not isinstance(params, dict):
        raise StyxValidationError(f'Params object has the wrong type \'{type(params)}\'')
    if params.get("key", None) is None:
        raise StyxValidationError("`key` must not be None")
    if not isinstance(params["key"], str):
        raise StyxValidationError(f'`key` has the wrong type: Received `{type(params.get("key", None))}` expected `str`')
    if params.get("value", None) is None:
        raise StyxValidationError("`value` must not be None")
    if not isinstance(params["value"], str):
        raise StyxValidationError(f'`value` has the wrong type: Received `{type(params.get("value", None))}` expected `str`')


def mrcolour_config_cargs(
    params: MrcolourConfigParamsDict,
    execution: Execution,
) -> list[str]:
    """
    Build command-line arguments from parameters.
    
    Args:
        params: The parameters.
        execution: The execution object for resolving input paths.
    Returns:
        Command-line arguments.
    """
    cargs = []
    cargs.append("-config")
    cargs.append(params.get("key", None))
    cargs.append(params.get("value", None))
    return cargs


class MrcolourOutputs(typing.NamedTuple):
    """
    Output object returned when calling `MrcolourParamsDict(...)`.
    """
    root: OutputPathType
    """Output root folder. This is the root folder for all outputs."""
    output: OutputPathType
    """the output image"""


def mrcolour_params(
    input_: InputPathType,
    map_: str,
    output: str,
    upper: float | None = None,
    lower: float | None = None,
    colour: list[float] | None = None,
    info: bool = False,
    quiet: bool = False,
    debug: bool = False,
    force: bool = False,
    nthreads: int | None = None,
    config: list[MrcolourConfigParamsDict] | None = None,
    help_: bool = False,
    version: bool = False,
) -> MrcolourParamsDictTagged:
    """
    Build parameters.
    
    Args:
        input_: the input image.
        map_: the colourmap to apply; choices are:\
            gray,hot,cool,jet,inferno,viridis,pet,colour,rgb.
        output: the output image.
        upper: manually set the upper intensity of the colour mapping.
        lower: manually set the lower intensity of the colour mapping.
        colour: set the target colour for use of the 'colour' map (three\
            comma-separated floating-point values).
        info: display information messages.
        quiet: do not display information messages or progress status;\
            alternatively, this can be achieved by setting the MRTRIX_QUIET\
            environment variable to a non-empty string.
        debug: display debugging messages.
        force: force overwrite of output files (caution: using the same file as\
            input and output might cause unexpected behaviour).
        nthreads: use this number of threads in multi-threaded applications\
            (set to 0 to disable multi-threading).
        config: temporarily set the value of an MRtrix config file entry.
        help_: display this information page and exit.
        version: display version information and exit.
    Returns:
        Parameter dictionary
    """
    params = {
        "@type": "mrtrix/mrcolour",
        "info": info,
        "quiet": quiet,
        "debug": debug,
        "force": force,
        "help": help_,
        "version": version,
        "input": input_,
        "map": map_,
        "output": output,
    }
    if upper is not None:
        params["upper"] = upper
    if lower is not None:
        params["lower"] = lower
    if colour is not None:
        params["colour"] = colour
    if nthreads is not None:
        params["nthreads"] = nthreads
    if config is not None:
        params["config"] = config
    return params


def mrcolour_validate(
    params: typing.Any,
) -> None:
    """
    Validate parameters. Throws an error if `params` is not a valid
    `MrcolourParamsDict` object.
    
    Args:
        params: The parameters object to validate.
    """
    if params is None or not isinstance(params, dict):
        raise StyxValidationError(f'Params object has the wrong type \'{type(params)}\'')
    if params.get("upper", None) is not None:
        if not isinstance(params["upper"], (float, int)):
            raise StyxValidationError(f'`upper` has the wrong type: Received `{type(params.get("upper", None))}` expected `float | None`')
    if params.get("lower", None) is not None:
        if not isinstance(params["lower"], (float, int)):
            raise StyxValidationError(f'`lower` has the wrong type: Received `{type(params.get("lower", None))}` expected `float | None`')
    if params.get("colour", None) is not None:
        if not isinstance(params["colour"], list):
            raise StyxValidationError(f'`colour` has the wrong type: Received `{type(params.get("colour", None))}` expected `list[float] | None`')
        for e in params["colour"]:
            if not isinstance(e, (float, int)):
                raise StyxValidationError(f'`colour` has the wrong type: Received `{type(params.get("colour", None))}` expected `list[float] | None`')
    if params.get("info", False) is None:
        raise StyxValidationError("`info` must not be None")
    if not isinstance(params["info"], bool):
        raise StyxValidationError(f'`info` has the wrong type: Received `{type(params.get("info", False))}` expected `bool`')
    if params.get("quiet", False) is None:
        raise StyxValidationError("`quiet` must not be None")
    if not isinstance(params["quiet"], bool):
        raise StyxValidationError(f'`quiet` has the wrong type: Received `{type(params.get("quiet", False))}` expected `bool`')
    if params.get("debug", False) is None:
        raise StyxValidationError("`debug` must not be None")
    if not isinstance(params["debug"], bool):
        raise StyxValidationError(f'`debug` has the wrong type: Received `{type(params.get("debug", False))}` expected `bool`')
    if params.get("force", False) is None:
        raise StyxValidationError("`force` must not be None")
    if not isinstance(params["force"], bool):
        raise StyxValidationError(f'`force` has the wrong type: Received `{type(params.get("force", False))}` expected `bool`')
    if params.get("nthreads", None) is not None:
        if not isinstance(params["nthreads"], int):
            raise StyxValidationError(f'`nthreads` has the wrong type: Received `{type(params.get("nthreads", None))}` expected `int | None`')
    if params.get("config", None) is not None:
        if not isinstance(params["config"], list):
            raise StyxValidationError(f'`config` has the wrong type: Received `{type(params.get("config", None))}` expected `list[MrcolourConfigParamsDict] | None`')
        for e in params["config"]:
            mrcolour_config_validate(e)
    if params.get("help", False) is None:
        raise StyxValidationError("`help` must not be None")
    if not isinstance(params["help"], bool):
        raise StyxValidationError(f'`help` has the wrong type: Received `{type(params.get("help", False))}` expected `bool`')
    if params.get("version", False) is None:
        raise StyxValidationError("`version` must not be None")
    if not isinstance(params["version"], bool):
        raise StyxValidationError(f'`version` has the wrong type: Received `{type(params.get("version", False))}` expected `bool`')
    if params.get("input", None) is None:
        raise StyxValidationError("`input` must not be None")
    if not isinstance(params["input"], (pathlib.Path, str)):
        raise StyxValidationError(f'`input` has the wrong type: Received `{type(params.get("input", None))}` expected `InputPathType`')
    if params.get("map", None) is None:
        raise StyxValidationError("`map` must not be None")
    if not isinstance(params["map"], str):
        raise StyxValidationError(f'`map` has the wrong type: Received `{type(params.get("map", None))}` expected `str`')
    if params.get("output", None) is None:
        raise StyxValidationError("`output` must not be None")
    if not isinstance(params["output"], str):
        raise StyxValidationError(f'`output` has the wrong type: Received `{type(params.get("output", None))}` expected `str`')


def mrcolour_cargs(
    params: MrcolourParamsDict,
    execution: Execution,
) -> list[str]:
    """
    Build command-line arguments from parameters.
    
    Args:
        params: The parameters.
        execution: The execution object for resolving input paths.
    Returns:
        Command-line arguments.
    """
    cargs = []
    cargs.append("mrcolour")
    if params.get("upper", None) is not None:
        cargs.extend([
            "-upper",
            str(params.get("upper", None))
        ])
    if params.get("lower", None) is not None:
        cargs.extend([
            "-lower",
            str(params.get("lower", None))
        ])
    if params.get("colour", None) is not None:
        cargs.extend([
            "-colour",
            ",".join(map(str, params.get("colour", None)))
        ])
    if params.get("info", False):
        cargs.append("-info")
    if params.get("quiet", False):
        cargs.append("-quiet")
    if params.get("debug", False):
        cargs.append("-debug")
    if params.get("force", False):
        cargs.append("-force")
    if params.get("nthreads", None) is not None:
        cargs.extend([
            "-nthreads",
            str(params.get("nthreads", None))
        ])
    if params.get("config", None) is not None:
        cargs.extend([a for c in [mrcolour_config_cargs(s, execution) for s in params.get("config", None)] for a in c])
    if params.get("help", False):
        cargs.append("-help")
    if params.get("version", False):
        cargs.append("-version")
    cargs.append(execution.input_file(params.get("input", None)))
    cargs.append(params.get("map", None))
    cargs.append(params.get("output", None))
    return cargs


def mrcolour_outputs(
    params: MrcolourParamsDict,
    execution: Execution,
) -> MrcolourOutputs:
    """
    Build outputs object containing output file paths and possibly stdout/stderr.
    
    Args:
        params: The parameters.
        execution: The execution object for resolving input paths.
    Returns:
        Outputs object.
    """
    ret = MrcolourOutputs(
        root=execution.output_file("."),
        output=execution.output_file(params.get("output", None)),
    )
    return ret


def mrcolour_execute(
    params: MrcolourParamsDict,
    runner: Runner | None = None,
) -> MrcolourOutputs:
    """
    mrcolour
    
    Apply a colour map to an image.
    
    Under typical usage, this command will receive as input ad 3D greyscale
    image, and output a 4D image with 3 volumes corresponding to red-green-blue
    components; other use cases are possible, and are described in more detail
    below.
    
    By default, the command will automatically determine the maximum and minimum
    intensities of the input image, and use that information to set the upper
    and lower bounds of the applied colourmap. This behaviour can be overridden
    by manually specifying these bounds using the -upper and -lower options
    respectively.
    
    References:
    
    .
    
    Author: MRTrix3 Developers
    
    URL: https://www.mrtrix.org/
    
    Args:
        params: The parameters.
        runner: Command runner.
    Returns:
        NamedTuple of outputs (described in `MrcolourOutputs`).
    """
    mrcolour_validate(params)
    runner = runner or get_global_runner()
    execution = runner.start_execution(MRCOLOUR_METADATA)
    params = execution.params(params)
    cargs = mrcolour_cargs(params, execution)
    ret = mrcolour_outputs(params, execution)
    execution.run(cargs)
    return ret


def mrcolour(
    input_: InputPathType,
    map_: str,
    output: str,
    upper: float | None = None,
    lower: float | None = None,
    colour: list[float] | None = None,
    info: bool = False,
    quiet: bool = False,
    debug: bool = False,
    force: bool = False,
    nthreads: int | None = None,
    config: list[MrcolourConfigParamsDict] | None = None,
    help_: bool = False,
    version: bool = False,
    runner: Runner | None = None,
) -> MrcolourOutputs:
    """
    mrcolour
    
    Apply a colour map to an image.
    
    Under typical usage, this command will receive as input ad 3D greyscale
    image, and output a 4D image with 3 volumes corresponding to red-green-blue
    components; other use cases are possible, and are described in more detail
    below.
    
    By default, the command will automatically determine the maximum and minimum
    intensities of the input image, and use that information to set the upper
    and lower bounds of the applied colourmap. This behaviour can be overridden
    by manually specifying these bounds using the -upper and -lower options
    respectively.
    
    References:
    
    .
    
    Author: MRTrix3 Developers
    
    URL: https://www.mrtrix.org/
    
    Args:
        input_: the input image.
        map_: the colourmap to apply; choices are:\
            gray,hot,cool,jet,inferno,viridis,pet,colour,rgb.
        output: the output image.
        upper: manually set the upper intensity of the colour mapping.
        lower: manually set the lower intensity of the colour mapping.
        colour: set the target colour for use of the 'colour' map (three\
            comma-separated floating-point values).
        info: display information messages.
        quiet: do not display information messages or progress status;\
            alternatively, this can be achieved by setting the MRTRIX_QUIET\
            environment variable to a non-empty string.
        debug: display debugging messages.
        force: force overwrite of output files (caution: using the same file as\
            input and output might cause unexpected behaviour).
        nthreads: use this number of threads in multi-threaded applications\
            (set to 0 to disable multi-threading).
        config: temporarily set the value of an MRtrix config file entry.
        help_: display this information page and exit.
        version: display version information and exit.
        runner: Command runner.
    Returns:
        NamedTuple of outputs (described in `MrcolourOutputs`).
    """
    params = mrcolour_params(
        upper=upper,
        lower=lower,
        colour=colour,
        info=info,
        quiet=quiet,
        debug=debug,
        force=force,
        nthreads=nthreads,
        config=config,
        help_=help_,
        version=version,
        input_=input_,
        map_=map_,
        output=output,
    )
    return mrcolour_execute(params, runner)


__all__ = [
    "MRCOLOUR_METADATA",
    "MrcolourConfigParamsDict",
    "MrcolourConfigParamsDictTagged",
    "MrcolourOutputs",
    "MrcolourParamsDict",
    "MrcolourParamsDictTagged",
    "mrcolour",
    "mrcolour_config",
    "mrcolour_execute",
    "mrcolour_params",
]
